<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * IG_ES_Cart_Item
 * 
 * @since 4.6.5
 */
class IG_ES_Cart_Item {

	/**
	 * Cart item key 
	 *
	 * @var string 
	 */
	protected $key;

	/** 
	 * Cart item data
	 *
	 * @var array 
	 */
	protected $data;


	/**
	 * IG_ES_Cart_Item constructor
	 * 
	 * @param string $key
	 * @param array $data
	 */
	public function __construct( $key, $data ) {
		$this->key = $key;
		$this->data = is_array( $data ) ? $data : array();
	}


	/**
	 * Method to get cart item product.
	 * 
	 * @return WC_Product
	 */
	public function get_product() {
		return wc_get_product( $this->get_variation_id() ? $this->get_variation_id() : $this->get_product_id() );
	}

	/**
	 * Method to get cart item product name.
	 * 
	 * @return string
	 */
	public function get_name() {
		$product = $this->get_product();
		if ( $product ) {
			return apply_filters( 'woocommerce_cart_item_name', $product->get_name(), $this->data, $this->key );
		}
		return '';
	}

	/**
	 * Method to get cart item key.
	 * 
	 * @return int
	 */
	public function get_key() {
		return $this->key;
	}


	/**
	 * Method to get cart item
	 * 
	 * @return array
	 */
	public function get_data() {
		return $this->data;
	}

	/**
	 * Method to get variation data for cart item.
	 * 
	 * @return array
	 */
	public function get_variation_data() {
		return isset( $this->data['variation'] ) && is_array( $this->data['variation'] ) ? $this->data['variation'] : array();
	}

	/**
	 * Method to get product id
	 * 
	 * @return int
	 */
	public function get_product_id() {
		return isset( $this->data['product_id'] ) ? ES_Clean::id( $this->data['product_id'] ) : 0;
	}


	/**
	 * Method to set product id
	 * 
	 * @param $id
	 */
	public function set_product_id( $id ) {
		$this->data['product_id'] = ES_Clean::id( $id );
	}

	/**
	 * Method to get prodcut variation id 
	 * 
	 * @return int
	 */
	public function get_variation_id() {
		return isset( $this->data['variation_id'] ) ? ES_Clean::id( $this->data['variation_id'] ) : 0;
	}


	/**
	 * Method to set product variation id 
	 * 
	 * @param $id
	 */
	public function set_variation_id( $id ) {
		$this->data['variation_id'] = ES_Clean::id( $id );
	}


	/**
	 * Method to get line subtotal
	 * 
	 * @return float
	 */
	public function get_line_subtotal() {
		return isset( $this->data['line_subtotal'] ) ? floatval( $this->data['line_subtotal'] ) : 0;
	}


	/**
	 * Method to get line subtotal tax
	 * 
	 * @return float
	 */
	public function get_line_subtotal_tax() {
		return isset( $this->data['line_subtotal_tax'] ) ? floatval( $this->data['line_subtotal_tax'] ) : 0;
	}

	/**
	 * Get item quantity.
	 *
	 * @return int|float
	 */
	public function get_quantity() {
		$quantity = wc_stock_amount( isset( $this->data['quantity'] ) ? $this->data['quantity'] : 0 );

		return apply_filters( 'ig_es_cart_item_get_quantity', $quantity, $this );
	}

	/**
	 * Gets and formats a list of cart item data + variations for display on the frontend.
	 *
	 * @param bool $flat (default: false)
	 * @return string
	 */
	public function get_item_data_html( $flat = false ) {
		$item_data = array();

		foreach ( $this->get_variation_data() as $name => $value ) {
			$taxonomy = wc_attribute_taxonomy_name( str_replace( 'attribute_pa_', '', urldecode( $name ) ) );

			// If this is a term slug, get the term's nice name
			if ( taxonomy_exists( $taxonomy ) ) {
				$term = get_term_by( 'slug', $value, $taxonomy );
				if ( ! is_wp_error( $term ) && $term && $term->name ) {
					$value = $term->name;
				}
				$label = wc_attribute_label( $taxonomy );

				// If this is a custom option slug, get the options name.
			} else {
				$value = apply_filters( 'woocommerce_variation_option_name', $value );
				$label = wc_attribute_label( str_replace( 'attribute_', '', $name ), $this->get_product() );
			}

			// Check the nicename against the title.
			if ( '' === $value || wc_is_attribute_in_product_name( $value, $this->get_name() ) ) {
				continue;
			}

			$item_data[] = array(
				'key'   => $label,
				'value' => $value,
			);
		}

		// Filter item data to allow 3rd parties to add more to the array
		$item_data = apply_filters( 'woocommerce_get_item_data', $item_data, $this->data );

		// Format item data ready to display
		foreach ( $item_data as $key => $data ) {
			// Set hidden to true to not display meta on cart.
			if ( ! empty( $data['hidden'] ) ) {
				unset( $item_data[ $key ] );
				continue;
			}
			$item_data[ $key ]['key']     = ! empty( $data['key'] ) ? $data['key'] : $data['name'];
			$item_data[ $key ]['display'] = ! empty( $data['display'] ) ? $data['display'] : $data['value'];
		}

		// Output flat or in list format
		if ( count( $item_data ) > 0 ) {
			ob_start();

			if ( $flat ) {
				foreach ( $item_data as $data ) {
					echo esc_html( wp_strip_all_tags( $data['key'] ) ) . ': ' . wp_kses_post( $data['display'] ) . "\n";
				}
			} else {
				wc_get_template( 'cart/cart-item-data.php', array( 'item_data' => $item_data ) );
			}

			return ob_get_clean();
		}

		return '';
	}
}
