<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Cart class
 */
class IG_ES_Cart extends IG_ES_DB_WC_Cart {

	/**
	 * Cart total
	 * 
	 * @var float 
	 */
	public $calculated_total = 0;

	/**
	 * Cart total tax
	 *
	 * @var float 
	 */
	public $calculated_tax_total = 0;

	/** 
	 * Cart subtotal
	 *
	 * @var float 
	 */
	public $calculated_subtotal = 0;

	/**
	 * Table name
	 *
	 * @since 4.6.5
	 * @var string $table_name
	 */
	public $table_name;

	/**
	 * Table version
	 *
	 * @since 4.6.5
	 * @var string $version
	 */
	public $version;

	/**
	 * Table primary key
	 *
	 * @since 4.6.5
	 * @var string
	 */
	public $primary_key;

	/**
	 * ID
	 *
	 * @since 4.6.5
	 * @var integer ID
	 */
	public $id = 0;

	/**
	 * Workflow queue data
	 *
	 * @since 4.6.5
	 * @var array
	 */
	public $data = array();

	/**
	 * Variable to store changed field in cart data
	 *
	 * @since 4.6.5
	 * @var array
	 */
	public $changed_fields = array();

	/**
	 * Flag to check whether cart is valid
	 *
	 * @since 4.6.5
	 * @var bool
	 */
	public $exists = false;

	/**
	 * Added Logger Context
	 *
	 * @since 4.2.0
	 * @var array
	 *
	 */
	public $logger_context = array(
		'source' => 'ig_es_wc_cart'
	);

	/**
	 * Class contructor
	 * 
	 * @param bool|int $id
	 */
	public function __construct( $id = false ) {
		parent::__construct();
		if ( $id ) {
			$this->get_by( 'id', $id );
		}
	}

	/**
	 * Method to get cart id
	 * 
	 * @return int
	 */
	public function get_id() {
		return $this->id ? (int) $this->id : 0;
	}

	/**
	 * Method to get cart by cart id
	 * 
	 * @param $value string|int
	 * @param $field string
	 */
	public function get_by_cart_id( $cart_id ) {

		global $wpdb;

		$wp_user = get_user_by( 'ID', $cart_id );

		return $wp_user;
	}


	/**
	 * Method to get cart status
	 * 
	 * @return string
	 */
	public function get_status() {
		$status = $this->get_prop( 'status' );
		return $status ? ES_Clean::string( $status ) : 'abandoned';
	}


	/**
	 * Method to set cart status 
	 * 
	 * @param $status - active|abandoned
	 */
	public function set_status( $status ) {
		$this->set_prop( 'status', ES_Clean::string( $status ) );
	}


	/**
	 * Transition status, triggers change hooks
	 * 
	 * @param $new_status - active|abandoned
	 */
	public function update_status( $new_status ) {

		$old_status = $this->get_status();

		if ( $new_status === $old_status ) {
			return;
		}

		$this->set_status( $new_status );
		$this->save();

		do_action( 'ig_es_wc_cart_status_changed', $this, $old_status, $new_status );
	}


	/**
	 * Set cart data option
	 * 
	 * @param $key
	 * @param $value
	 */
	public function set_prop( $key, $value ) {

		if ( is_array( $value ) && ! $value ) {
			$value = ''; // convert empty arrays to blank
		}

		$this->data[$key] = $value;
		$this->changed_fields[] = $key;
	}

	/**
	 * Method to check if cart has given data
	 * 
	 * @param $key
	 * @return bool
	 */
	public function has_prop( $key ) {
		return isset( $this->data[$key] );
	}

	/**
	 * Get cart data option
	 * 
	 * @param $key
	 * @return mixed
	 */
	public function get_prop( $key ) {
		if ( ! isset( $this->data[$key] ) ) {
			return false;
		}

		$value = $this->data[$key];
		$value = maybe_unserialize( $value );

		return $value;
	}

	/**
	 * Method to get user id
	 * 
	 * @return int
	 */
	public function get_user_id() {
		return ES_Clean::id( $this->get_prop( 'user_id' ) );
	}

	/**
	 * Method to set user id 
	 * 
	 * @param int $user_id
	 */
	public function set_user_id( $user_id ) {
		$this->set_prop( 'user_id', ES_Clean::id( $user_id ) );
	}

	/**
	 * Get guest ID
	 * 
	 * @return int
	 */
	public function get_guest_id() {
		return ES_Clean::id( $this->get_prop( 'guest_id' ) );
	}


	/**
	 * Set guest ID
	 * 
	 * @param int $guest_id
	 */
	public function set_guest_id( $guest_id ) {
		$this->set_prop( 'guest_id', ES_Clean::id( $guest_id ) );
	}

	/**
	 * Method to set last modified date
	 * 
	 * @param DateTime|string $date
	 */
	public function set_date_last_modified( $date ) {
		$this->set_date_column( 'last_modified', $date );
	}


	/**
	 * Method to get creation date
	 * 
	 * @return bool|DateTime
	 */
	public function get_date_created() {
		return $this->get_date_column( 'created' );
	}


	/**
	 * Method to set creation date 
	 * 
	 * @param DateTime $date
	 */
	public function set_date_created( $date ) {
		$this->set_date_column( 'created', $date );
	}


	/**
	 * Method to get cart total
	 * 
	 * @return float
	 */
	public function get_total() {
		return (float) $this->get_prop( 'total' );
	}


	/**
	 * Method to set cart total 
	 * 
	 * @param $total
	 */
	public function set_total( $total ) {
		$this->set_prop( 'total', wc_format_decimal( $total ) );
	}


	/**
	 * Method to set shipping total 
	 * 
	 * @param $val
	 */
	public function set_shipping_total( $val ) {
		$this->set_prop( 'shipping_total', wc_format_decimal( $val ) );
	}


	/**
	 * Method to set shipping total tax
	 * 
	 * @param $val
	 */
	public function set_shipping_tax_total( $val ) {
		$this->set_prop( 'shipping_tax_total', wc_round_tax_total( $val ) );
	}

	/**
	 * Method to get shipping total charge 
	 * 
	 * @return float
	 */
	public function get_shipping_total() {
		return (float) $this->get_prop( 'shipping_total' );
	}


	/**
	 * Method to get shipping total
	 * 
	 * @return float
	 */
	public function get_shipping_tax_total() {
		return (float) $this->get_prop( 'shipping_tax_total' );
	}


	/**
	 * Method to get cart token
	 * 
	 * @return string
	 */
	public function get_token() {
		return ES_Clean::string( $this->get_prop( 'token' ) );
	}
	
	/**
	 * Method to set cart token
	 * 
	 * @param bool|string $token (optional)
	 */
	public function set_token( $token = false ) {
		if ( ! $token ) {
			$token = ig_es_generate_key( 32 );
		}

		$this->set_prop( 'token', ES_Clean::string( $token ) );
	}
	
	/**
	 * Method to get cart restore url
	 * 
	 * @param $cart Cart
	 * @param $parameters array
	 * @return string
	 */
	public function get_cart_url() {

		$cart_url = add_query_arg( array(
			'es'    => 'restore-wc-cart',
			'token' => $this->get_token(),
		), wc_get_page_permalink( 'cart' ) );

		return $cart_url;
	}

	/**
	 * Method to get cart currency
	 * 
	 * @return float
	 */
	public function get_currency() {
		$currency = $this->get_prop( 'currency' );
		if ( $currency ) {
			return ES_Clean::string( $currency );
		}
		return get_woocommerce_currency();
	}


	/**
	 * Method to set cart currency
	 * 
	 * @param $currency
	 */
	public function set_currency( $currency ) {
		$this->set_prop( 'currency', ES_Clean::string( $currency ) );
	}

	/**
	 * Method to get shipping total html
	 * 
	 * @return string
	 */
	public function get_shipping_total_html() {
		$total = get_option( 'woocommerce_tax_display_cart' ) === 'excl' ? $this->get_shipping_total() : $this->get_shipping_total() + $this->get_shipping_tax_total();

		if ( 0 === $total ) {
			$html = __( 'Free!', 'email-subscribers' );
		} else {
			$html = $this->price( $total );
		}

		return apply_filters( 'ig_es_cart_get_shipping_total_html', $html, $this );
	}


	/**
	 * Method to check if shipping is needed
	 * 
	 * @return bool
	 */
	public function needs_shipping() {

		if ( ! wc_shipping_enabled() || 0 === wc_get_shipping_method_count( true ) ) {
			return false;
		}

		$needs_shipping = false;

		if ( $this->has_items() ) {
			foreach ( $this->get_items() as $cart_item ) {
				$product = $cart_item->get_product();
				if ( $product && $product->needs_shipping() ) {
					$needs_shipping = true;
					break;
				}
			}
		}

		return $needs_shipping;
	}


	/**
	 * Method to check if cart has coupons 
	 * 
	 * @return bool
	 */
	public function has_coupons() {
		return count( $this->get_coupons() ) > 0;
	}


	/**
	 * Method to get applied coupons in the cart
	 * 
	 * @return array
	 */
	public function get_coupons() {
		$coupons = $this->get_prop( 'coupons' );
		return is_array( $coupons ) ? $coupons : array();
	}


	/**
	 * Method to set coupons in the cart 
	 * 
	 * @param array $coupon_data
	 */
	public function set_coupons( $coupon_data ) {
		$this->set_prop( 'coupons', (array) $coupon_data );
	}


	/**
	 * Method to get fees
	 * 
	 * @return array
	 */
	public function get_fees() {
		$fees = $this->get_prop( 'fees' );
		return is_array( $fees ) ? $fees : array();
	}


	/**
	 * Method to set fees in the cart
	 * 
	 * @param array $fees_data
	 */
	public function set_fees( $fees_data ) {
		$this->set_prop( 'fees', (array) $fees_data );
	}


	/**
	 * Method to check if cart has any items
	 * 
	 * @return bool
	 */
	public function has_items() {
		$items = $this->get_prop( 'items' );
		return is_array( $items ) && count( $items ) > 0;
	}


	/**
	 * Method to get cart items
	 * 
	 * @return IG_ES_Cart_Item[]
	 */
	public function get_items() {
		$items = $this->get_prop( 'items' );

		if ( is_array( $items ) ) {
			foreach ( $items as $item_key => $item_data ) {
				$items[ $item_key ] = new IG_ES_Cart_Item( $item_key, $item_data );
			}
		} else {
			$items = array();
		}

		return apply_filters( 'ig_es_cart_get_items', $items );
	}


	/**
	 * Method to get cart item's data in serialized format
	 * 
	 * @return array
	 */
	public function get_items_raw() {
		$raw = array();
		foreach ( $this->get_items() as $item ) {
			$raw[ $item->get_key() ] = $item->get_data();
		}
		return $raw;
	}


	/**
	 * Method to get cart item count
	 * 
	 * @return int
	 */
	public function get_item_count() {
		$count = 0;

		foreach ( $this->get_items() as $item ) {
			$count += $item->get_quantity();
		}

		return apply_filters( 'ig_es_cart_get_item_count', $count, $this->get_items() );
	}

	/**
	 * Method to set cart items.
	 * 
	 * @param array $items
	 */
	public function set_items( $items ) {
		$this->set_prop( 'items', (array) $items );
	}

	/**
	 * Get guest
	 * 
	 * @return IG_ES_Guest|false
	 */
	public function get_guest() {
		if ( ! $this->get_guest_id() ) {
			return false;
		}
		return IG_ES_Guest_Factory::get( $this->get_guest_id() );
	}

	/**
	 * Method to get customer 
	 * 
	 * @return IG_ES_Customer|bool
	 */
	public function get_customer() {
		if ( $this->get_user_id() ) {
			return IG_ES_Customer_Factory::get_by_user_id( $this->get_user_id() );
		} else {
			return IG_ES_Customer_Factory::get_by_guest_id( $this->get_guest_id() );
		}
	}

	/**
	 * Method get cart data by column value
	 * 
	 * @param $column string|int
	 * @param $row_id string
	 */
	public function get_by( $column, $column_value, $output = ARRAY_A, $use_cahe = false ) {

		$row = parent::get_by( $column, $column_value, $output );

		if ( ! $row ) {
			return;
		}

		if ( ! empty( $row['id'] ) ) {
			$this->id = $row['id'];
		}

		$this->fill( $row );
	}

	/**
	 * Fill model with data
	 *
	 * @param array $row
	 */
	public function fill( $row ) {
		if ( ! is_array( $row ) ) {
			return;
		}

		$this->data = $row;
		$this->original_data = $row;
		$this->exists = true;

		do_action( 'ig_es_object_load', $this );
	}

	/**
	 * Updates the stored cart with the current time and cart items
	 */
	public function sync() {

		$this->set_date_last_modified( new DateTime() );
		$this->set_items( WC()->cart->get_cart_for_session() );

		$coupon_data = array();

		foreach ( WC()->cart->get_applied_coupons() as $coupon_code ) {
			$coupon_data[$coupon_code] = array(
				'discount_incl_tax' => WC()->cart->get_coupon_discount_amount( $coupon_code, false ),
				'discount_excl_tax' => WC()->cart->get_coupon_discount_amount( $coupon_code ),
				'discount_tax' => WC()->cart->get_coupon_discount_tax_amount( $coupon_code )
			);
		}

		$this->set_coupons( $coupon_data );
		$this->set_fees( WC()->cart->get_fees() );
		$this->set_currency( get_woocommerce_currency() );
		$this->set_shipping_tax_total( WC()->cart->shipping_tax_total );
		$this->set_shipping_total( WC()->cart->shipping_total );

		$this->calculate_totals();

		$this->set_total( $this->calculated_total );

		if ( 'abandoned' === $this->get_status()  ) {
			$this->update_status( 'active' );
		} else {
			$this->save();
		}
	}


	public function calculate_totals() {

		$this->calculated_subtotal = 0;
		$this->calculated_tax_total = 0;
		$this->calculated_total = 0;

		$tax_display = get_option( 'woocommerce_tax_display_cart' );

		foreach ( $this->get_items() as $item ) {
			$this->calculated_tax_total += $item->get_line_subtotal_tax();
			$this->calculated_total += $item->get_line_subtotal() + $item->get_line_subtotal_tax();
			$this->calculated_subtotal += 'excl' === $tax_display ? $item->get_line_subtotal() : $item->get_line_subtotal() + $item->get_line_subtotal_tax();
		}

		foreach ( $this->get_coupons() as $coupon_code => $coupon ) {
			$this->calculated_total -= $coupon[ 'discount_incl_tax' ];
			$this->calculated_tax_total -= $coupon['discount_tax'];
		}

		foreach ( $this->get_fees() as $fee ) {
			$this->calculated_total += ( $fee->amount + $fee->tax );
			$this->calculated_tax_total += $fee->tax;
		}

		$this->calculated_tax_total += $this->get_shipping_tax_total();
		$this->calculated_total += $this->get_shipping_total();
		$this->calculated_total += $this->get_shipping_tax_total();
	}


	/**
	 * Method to convert price into WC Price format
	 * 
	 * @param float $price
	 * @return string
	 */
	public function price( $price ) {
		return wc_price( $price, apply_filters( 'ig_es_cart_price_args', array(), $this ) );
	}

	/**
	 * Sets the value of a date column from  a mixed input.
	 *
	 * $value can be an instance of WC_DateTime the timezone will be ignored.
	 * If $value is a string it must be MYSQL formatted.
	 *
	 * @param string                                 $column
	 * @param DateTime|\DateTime|string $value
	 */
	protected function set_date_column( $column, $value ) {
		if ( is_a( $value, 'DateTime' ) ) {
			// convert to UTC time
			$utc_date = new DateTime();
			$utc_date->setTimestamp( $value->getTimestamp() );
			$this->set_prop( $column, $utc_date->format( 'Y-m-d H:i:s' ) );
		} elseif ( $value ) {
			$this->set_prop( $column, ES_Clean::string( $value ) );
		}
	}

	/**
	 * Set cart creation date/time
	 * 
	 * @param DateTime $date
	 */
	public function set_created( $date ) {
		$this->set_date_column( 'created', $date );
	}

	/**
	 * Inserts or updates the model
	 * Only updates modified fields
	 *
	 * @return bool True on success, false on error.
	 */
	public function save() {

		if ( $this->exists ) {
			// update changed fields
			$changed_data = array_intersect_key( $this->data, array_flip( $this->changed_fields ) );

			// serialize
			$changed_data = array_map( 'maybe_serialize', $changed_data );

			if ( empty( $changed_data ) ) {
				return true;
			}

			$queue_id = $this->get_id();

			$updated = $this->update( $queue_id, $changed_data );

			if ( false === $updated ) {
				// Return here to prevent cache updates on error
				return false;
			}

			do_action( 'ig_es_object_update', $this ); // cleans object cache
		} else {
			$this->set_created( new DateTime() );
			$this->data = array_map( 'maybe_serialize', $this->data );
			
			// insert row
			$queue_id = $this->insert( $this->data );

			if ( $queue_id ) {
				$this->exists = true;
				$this->id = $queue_id;
			} else {
				/* translators: %s: Table name */
				ES()->logger->error( sprintf( __( 'Could not insert into \'%1$s\' table. \'%1$s\' may not be present in the database.', 'email-subscribers' ), $this->table_name ), $this->logger_context );

				// Return here to prevent cache updates on error
				return false;
			}
		}

		// reset changed data
		// important to reset after cache hooks
		$this->changed_fields = array();
		$this->original_data = $this->data;

		return true;
	}

	/**
	 * Delete cart
	 * 
	 * @return void
	 */
	public function delete_cart() {
		$cart_id = $this->id;
		$this->delete( $cart_id );
	}

	/**
	 * Set cart ID
	 * @param $id
	 */
	public function set_id( $id ) {
		$this->id = $id;
	}

}
