<?php

/**
 * Customer factory class
 * 
 * @since 4.6.5
 */
class IG_ES_Customer_Factory extends IG_ES_Factory {

	public static $model = 'IG_ES_Customer';

	/**
	 * Method to get customer id
	 * 
	 * @param int $customer_id
	 * @return IG_ES_Customer|bool
	 */
	public static function get( $customer_id ) {
		return parent::get( $customer_id );
	}


	/**
	 * Method to get customer by user id
	 * 
	 * @param int $user_id
	 * @param bool $create - create the customer record if it doesn't exist
	 * @return IG_ES_Customer|bool
	 */
	public static function get_by_user_id( $user_id, $create = true ) {

		if ( ! $user_id ) {
			return false;
		}

		$user_id = ES_Clean::id( $user_id );

		if ( ES_Cache::is_exists( $user_id, 'customer_user_id' ) ) {
			return static::get( ES_Cache::get( $user_id, 'customer_user_id' ) );
		}

		$customer = new IG_ES_Customer( $user_id );

		if ( $customer->exists ) {
			return $customer;
		}

		return false;
	}

	/**
	 * Get customer by guest ID.
	 * 
	 * @param int $guest_id
	 * @param bool $create - create the customer record if it doesn't exist
	 * @return IG_ES_Customer|bool
	 */
	public static function get_by_guest_id( $guest_id, $create = true ) {

		if ( ! $guest_id ) {
			return false;
		}

		$guest_id = ES_Clean::id( $guest_id );

		if ( ES_Cache::is_exists( $guest_id, 'customer_guest_id' ) ) {
			return static::get( ES_Cache::get( $guest_id, 'customer_guest_id' ) );
		}

		$customer = new IG_ES_Customer();
		$customer->get_by( 'guest_id', $guest_id );

		if ( $customer->exists ) {
			return $customer;
		}
		
		if ( $create ) {
			// attempt to create the customer
			$customer = static::create_from_guest( $guest_id );

			if ( $customer ) {
				return $customer;
			}
		}

		return false;
	}

	/**
	 * Create customer object from guest ID.
	 * 
	 * @param int $guest_id
	 * @return IG_ES_Customer|bool
	 */
	public static function create_from_guest( $guest_id ) {

		$guest = IG_ES_Guest_Factory::get( $guest_id );

		if ( ! $guest ) {
			return false;
		}

		$customer = new IG_ES_Customer();
		$customer->set_prop( 'guest_id', $guest_id );
		$customer->set_key( static::generate_unique_customer_key() );
		$customer->exists = true;

		return $customer;
	}

	/**
	 * Get customer by email
	 * 
	 * @param string $email
	 * @param bool $create - create the customer record if it doesn't exist
	 * @return IG_ES_Customer|bool
	 */
	public static function get_by_email( $email, $create = true ) {

		if ( ! is_email( $email ) ) {
			return false;
		}

		$email = ES_Clean::email( $email );

		// check for matching user
		$user = get_user_by( 'email', $email );
		if ( $user ) {
			return static::get_by_user_id( $user->ID, $create );
		}

		// check for matching guest
		$guest = IG_ES_Guest_Factory::get_by_email( $email );
		if ( $guest ) {
			return static::get_by_guest_id( $guest->get_id(), $create );
		}

		if ( $create ) {
			// create guest for new customer
			$guest = IG_ES_Guest_Factory::create( $email );
			if ( $guest instanceof IG_ES_Guest ) {
				$customer = new IG_ES_Customer();
				$customer->set_prop( 'guest_id', $guest->get_id() );
				$customer->exists = true;
				return $customer;
			}

			return static::get_by_guest_id( $guest->get_id() );
		}

		return false;
	}

	/**
	 * Generate unique tracking key for customer
	 * 
	 * @return string
	 */
	public static function generate_unique_customer_key() {

		$key = ig_es_generate_key(20, false );

		return $key;
	}

	/**
	 * Method to update customer cache.
	 * 
	 * @param IG_ES_Customer $customer
	 */
	public static function update_cache( $customer ) {
		parent::update_cache( $customer );

		if ( $customer->get_user_id() ) {
			ES_Cache::set( $customer->get_user_id(), $customer->get_id(), 'customer_user_id' );
		}

		ES_Cache::set( $customer->get_key(), $customer->get_id(), 'customer_key' );
	}

	/**
	 * Method to clean customer cache.
	 * 
	 * @param IG_ES_Customer $customer
	 */
	public static function clean_cache( $customer ) {
		parent::clean_cache( $customer );

		static::clear_cached_prop( $customer, 'user_id', 'customer_user_id' );
		static::clear_cached_prop( $customer, 'id_key', 'customer_key' );
	}
}
