<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Class for WooCommerce Customer
 * 
 * @since 4.6.5
 */
class IG_ES_Customer {

	/**
	 * Customer ID
	 *
	 * @since 4.6.5
	 * @var integer Customer ID
	 */
	public $id = 0;

	/**
	 * Customer queue data
	 *
	 * @since 4.6.5
	 * @var array
	 */
	public $data = array();

	/**
	 * Flag to check whether Customer queue is valid
	 *
	 * @since 4.6.5
	 * @var bool
	 */
	public $exists = false;

	/**
	 * Constructor
	 *
	 * @param bool|int $id
	 */
	public function __construct( $user_id = false ) {
		if ( is_numeric( $user_id ) ) {
			$wp_user = $this->get_by_user_id( $user_id );
			if ( $wp_user instanceof WP_User ) {
				$this->id              = $user_id;
				$this->data            = (array) $wp_user;
				$this->data['user_id'] = $user_id;
				$this->exists          = true;
			}
		}
	}

	/**
	 * Method to get customer by user id
	 * 
	 * @param $value string|int
	 * @param $field string
	 * 
	 * @return WP_User|false
	 */
	public function get_by_user_id( $user_id ) {

		global $wpdb;

		$wp_user = get_user_by( 'ID', $user_id );

		return $wp_user;
	}

	/**
	 * Method to get customer id
	 * 
	 * @return int
	 */
	public function get_id() {
		return $this->id ? (int) $this->id : 0;
	}

	public function get_by( $meta_key = '', $meta_value = '' ) {
		
		if ( ! empty( $meta_key ) ) {
			$user_args = array(
				'meta_key'   => $meta_key, 
				'meta_value' => $meta_value,
			);
			$users = get_users( $user_args );
			if ( ! empty( $users ) ) {
				$wp_user	= array_shift( $users );
				if ( $wp_user instanceof WP_User ) {
					$this->id              = $wp_user->ID;
					$this->data            = (array) $wp_user;
					$this->data['user_id'] = $wp_user->ID;
					$this->exists          = true;
				}
			}
		}
	}

	/**
	 * Method to get user id
	 * 
	 * @return int
	 */
	public function get_user_id() {
		return (int) $this->get_prop( 'user_id' );
	}


	/**
	 * Method to set user id
	 * 
	 * @param $user_id
	 */
	public function set_user_id( $user_id ) {
		$this->set_prop( 'user_id', (int) $user_id );
	}

	/**
	 * Get guest id
	 * 
	 * @return int
	 */
	public function get_guest_id() {
		return (int) $this->get_prop( 'guest_id' );
	}

	/**
	 * Returns a unique key that can ID the customer. This is added to the customer upon creation.
	 *
	 * @return string
	 */
	public function get_key() {
		return ES_Clean::string( $this->get_prop( 'id_key' ) );
	}


	/**
	 * Set ID key
	 * 
	 * @param string $key
	 */
	public function set_key( $key ) {
		$this->set_prop( 'id_key', ES_Clean::string( $key ) );
	}

	/**
	 * Get Customer queue data option
	 * 
	 * @param $key
	 * @return mixed
	 */
	public function get_prop( $key ) {
		if ( ! isset( $this->data[$key] ) ) {
			return false;
		}

		$value = $this->data[$key];
		$value = maybe_unserialize( $value );

		return $value;
	}

	/**
	 * Set Customer queue data option
	 * 
	 * @param $key
	 * @param $value
	 */
	public function set_prop( $key, $value ) {

		if ( is_array( $value ) && ! $value ) {
			$value = ''; // convert empty arrays to blank
		}

		$this->data[$key] = $value;
		$this->changed_fields[] = $key;
	}

	/**
	 * Get guest
	 * 
	 * @return IG_ES_Guest|false
	 */
	public function get_guest() {
		if ( $this->is_registered() ) {
			return false;
		}
		return IG_ES_Guest_Factory::get( $this->get_guest_id() );
	}

	/**
	 * Get WP User
	 * 
	 * @return WP_User
	 */
	public function get_user() {
		return get_userdata( $this->get_user_id() );
	}


	/**
	 * Get cart object
	 * 
	 * @return IG_ES_Cart
	 */
	public function get_cart() {
		if ( $this->is_registered() ) {
			return IG_ES_Cart_Factory::get_by_user_id( $this->get_user_id() );
		} else {
			return IG_ES_Cart_Factory::get_by_guest_id( $this->get_guest_id() );
		}
	}

	/**
	 * Deletes the customer's stored cart.
	 *
	 * @since 4.6.5
	 */
	public function delete_cart() {
		$cart = $this->get_cart();
		if ( $cart ) {
			$cart->delete();
		}
	}


	/**
	 * Is user registered
	 * 
	 * @return bool
	 */
	public function is_registered() {
		return 0 !== $this->get_user_id();
	}

	/**
	 * Get customer email
	 * 
	 * @return string
	 */
	public function get_email() {
		return ES_Clean::email( $this->get_linked_prop( 'email' ) );
	}

	/**
	 * Get customer first name
	 * 
	 * @return string
	 */
	public function get_first_name() {
		return $this->get_linked_prop( 'first_name' );
	}

	/**
	 * Get customer last name 
	 * 
	 * @return string
	 */
	public function get_last_name() {
		return $this->get_linked_prop( 'last_name' );
	}

	/**
	 * Get customer full name
	 * 
	 * @return string
	 */
	public function get_full_name() {
		/* translators: 1. Customer first name 2. Customer last name */
		return trim( sprintf( _x( '%1$s %2$s', 'full name', 'email-subscribers' ), $this->get_first_name(), $this->get_last_name() ) );
	}

	/**
	 * Get customer billing country
	 * 
	 * @return string
	 */
	public function get_billing_country() {
		return $this->get_linked_prop( 'billing_country' );
	}


	/**
	 * Get customer billing state
	 * 
	 * @return string
	 */
	public function get_billing_state() {
		return $this->get_linked_prop( 'billing_state' );
	}


	/**
	 * Get customer billing phone
	 * 
	 * @return string
	 */
	public function get_billing_phone() {
		return $this->get_linked_prop( 'billing_phone' );
	}


	/**
	 * Get customer billing postcode
	 * 
	 * @return string
	 */
	public function get_billing_postcode() {
		return $this->get_linked_prop( 'billing_postcode' );
	}


	/**
	 * Get customer billing city
	 * 
	 * @return string
	 */
	public function get_billing_city() {
		return $this->get_linked_prop( 'billing_city' );
	}

	/**
	 * Get customer billing address 1
	 * 
	 * @return string
	 */
	public function get_billing_address_1() {
		return $this->get_linked_prop( 'billing_address_1' );
	}

	/**
	 * Get customer billing address 2
	 * 
	 * @return string
	 */
	public function get_billing_address_2() {
		return $this->get_linked_prop( 'billing_address_2' );
	}

	/**
	 * Get customer billing company
	 * 
	 * @return string
	 */
	public function get_billing_company() {
		return $this->get_linked_prop( 'billing_company' );
	}

	/**
	 * Get linked property
	 * 
	 * @param $prop
	 * @return mixed
	 */
	public function get_linked_prop( $prop ) {

		$guest = false;
		$user  = false;

		if ( $this->is_registered() ) {
			$user = $this->get_user();
			if ( ! $user ) {
				return false;
			}
		} else {
			$guest = $this->get_guest();
			if ( ! $guest ) {
				return false;
			}
		}

		switch ( $prop ) {
			case 'email':
				return $this->is_registered() ? $user->user_email : $guest->get_email();
				break;
			case 'first_name':
				return $this->is_registered() ? $user->billing_first_name : $guest->get_first_name();
				break;
			case 'last_name':
				return $this->is_registered() ? $user->billing_last_name : $guest->get_last_name();
				break;
			case 'billing_country':
				return $this->is_registered() ? $user->billing_country : $guest->get_country();
				break;
			case 'billing_state':
				return $this->is_registered() ? $user->billing_state : $guest->get_state();
				break;
			case 'billing_phone':
				return $this->is_registered() ? $user->billing_phone : $guest->get_phone();
				break;
			case 'billing_company':
				return $this->is_registered() ? $user->billing_company : $guest->get_company();
				break;
			case 'billing_address_1':
				return $this->is_registered() ? $user->billing_address_1 : $guest->get_address_1();
				break;
			case 'billing_address_2':
				return $this->is_registered() ? $user->billing_address_2 : $guest->get_address_2();
				break;
			case 'billing_postcode':
				return $this->is_registered() ? $user->billing_postcode : $guest->get_postcode();
				break;
			case 'billing_city':
				return $this->is_registered() ? $user->billing_city : $guest->get_city();
				break;
			case 'tracking_key':
				if ( $this->is_registered() ) {
					$key = get_user_meta( $this->get_user_id(), 'ig_es_visitor_key', true );
					// not every registered user will have a key
					if ( ! $key ) {
						$key = ig_es_generate_key( 32 );
						update_user_meta( $this->get_user_id(), 'ig_es_visitor_key', $key );
					}
					return $key;
				} else {
					// guests are always created with a key
					return $guest->get_key();
				}
				break;
		}
	}

}

