<?php
/**
 * Absctract factory class
 * 
 * @class IG_ES_Factory
 */
abstract class IG_ES_Factory {

	/** 
	 * Cached object
	 *
	 * @var array - must NOT be declared in child class  */
	public static $cache = array();

	/** 
	 * Model object
	 * 
	 * @var string - must be declared in child class */
	public static $model;


	/**
	 * Fetches the object type from factories array
	 * 
	 * @return string
	 */
	public static function get_object_type() {

		$class = get_called_class();
		$factories = array_flip( IG_ES_Factories::get_factories() );

		if ( ! isset( $factories[ $class ] ) ) {
			return false;
		}

		return $factories[ $class ];
	}


	/**
	 * Get object from id
	 * 
	 * @param integer $object_id
	 * @return ES_DB|bool|mixed
	 */
	public static function get( $object_id ) {
		$object_id = ES_Clean::id( $object_id );

		if ( ! $object_id ) {
			return false;
		}

		if ( static::is_cached( $object_id ) ) {
			return static::get_cached( $object_id );
		}

		/**
		 * Model object
		 *  
		 * @var ES_DB $object 
		 **/
		$object = new static::$model( $object_id );

		if ( ! $object || ! $object->exists ) {
			static::cache_nonexistent_object( $object_id );
			return false;
		}

		return $object;
	}


	/**
	 * Load object
	 * 
	 * @deprecated
	 * @param $object
	 * @return ES_DB|bool
	 */
	public static function load( $object ) {
		return $object;
	}



	/**
	 * Setup cache array for type
	 */
	public static function setup_cache() {
		if ( ! isset( self::$cache[ static::get_object_type() ] ) ) {
			self::$cache[ static::get_object_type() ] = array();
		}
	}


	/**
	 * Does the object existing the cache, returns true if the object is false in the cache
	 * 
	 * @param $object_id
	 * @return bool
	 */
	public static function is_cached( $object_id ) {
		static::setup_cache();
		return isset( self::$cache[ static::get_object_type() ][ $object_id ] );
	}


	/**
	 * Get cached object
	 * 
	 * @param $object_id
	 * @return bool|ES_DB
	 */
	public static function get_cached( $object_id ) {
		static::setup_cache();

		if ( ! isset( self::$cache[ static::get_object_type() ][$object_id] ) ) {
			return false;
		}

		return static::$cache[ static::get_object_type() ][ $object_id ];
	}


	/**
	 * Cache nonexistent cache
	 * 
	 * Cache the fact that the object does not exist
	 *
	 * @param $object_id
	 */
	public static function cache_nonexistent_object( $object_id ) {
		static::setup_cache();
		self::$cache[ static::get_object_type() ][ $object_id ] = false;
	}


	/**
	 * Update cache
	 * 
	 * @param ES_DB $object
	 */
	public static function update_cache( $object ) {
		static::setup_cache();

		if ( ! is_a( $object, 'ES_DB' ) ) {
			return;
		}

		static::$cache[ static::get_object_type() ][ $object->get_id() ] = $object;
	}


	/**
	 * Clean cache
	 * 
	 * @param ES_DB $object
	 */
	public static function clean_cache( $object ) {
		static::setup_cache();

		if ( ! is_a( $object, 'ES_DB' ) ) {
			return;
		}

		if ( isset( self::$cache[ static::get_object_type() ][ $object->get_id() ] ) ) {
			unset( self::$cache[ static::get_object_type() ][ $object->get_id() ] );
		}

	}


	/**
	 * Clears cache property for object based on new and existing values
	 *
	 * @param ES_DB $object
	 * @param string $prop
	 * @param string $group
	 */
	public static function clear_cached_prop( $object, $prop, $group ) {
		if ( isset( $object->original_data[$prop] ) ) {
			ES_Cache::delete( $object->original_data[$prop], $group ); // clear old value, important if the value has changed
		}

		if ( isset( $object->data[$prop] ) ) {
			ES_Cache::delete( $object->data[$prop], $group ); // clear for new value, important for newly created carts for example
		}
	}

}
