<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Class to store guest data
 * 
 * @class IG_ES_Guest
 *
 */
class IG_ES_Guest  extends IG_ES_DB_WC_Guest {

	/**
	 * Guest ID
	 *
	 * @since 4.6.5
	 * @var integer Guest ID
	 */
	public $id = 0;

	/**
	 * Guest queue data
	 *
	 * @var array
	 */
	public $data = array();

	/**
	 * Variable to store changed field in cart data
	 *
	 * @since 4.6.5
	 * @var array
	 */
	public $changed_fields = array();

	/**
	 * Added Logger Context
	 *
	 * @var array
	 *
	 */
	public $logger_context = array(
		'source' => 'ig_es_wc_guests'
	);

	/**
	 * Flag to check whether Customer queue is valid
	 *
	 * @var bool
	 */
	public $exists = false;

	/**
	 * Class constructor
	 * 
	 * @param $id
	 */
	public function __construct( $id = false ) {
		parent::__construct();
		if ( $id ) {
			$this->get_by( 'id', $id );
		}
	}
	
	/**
	 * Method to get cart id
	 * 
	 * @return int
	 */
	public function get_id() {
		return $this->id ? (int) $this->id : 0;
	}

	/**
	 * Method get cart data by column value
	 * 
	 * @param $column string|int
	 * @param $row_id string
	 */
	public function get_by( $column, $column_value, $output = ARRAY_A, $use_cache = false ) {

		$row = parent::get_by( $column, $column_value, $output );

		if ( ! $row ) {
			return;
		}

		if ( ! empty( $row['id'] ) ) {
			$this->id = $row['id'];
		}

		$this->fill( $row );
	}

	/**
	 * Fill model with data
	 *
	 * @param array $row
	 */
	public function fill( $row ) {
		if ( ! is_array( $row ) ) {
			return;
		}

		if ( ! empty( $row['meta'] ) ) {
			$row['meta'] = maybe_unserialize( $row['meta'] );
		}

		$this->data          = $row;
		$this->original_data = $row;
		$this->exists        = true;

		do_action( 'ig_es_object_load', $this );
	}

	/**
	 * Set guest email
	 * 
	 * @param string $email
	 */
	public function set_email( $email ) {
		$this->set_prop( 'email', ES_Clean::string( $email ) ); // clean as string for anonymized emails
	}


	/**
	 * Get guest email
	 * 
	 * @return string
	 */
	public function get_email() {
		return ES_Clean::string( $this->get_prop( 'email' ) ); // clean as string for anonymized emails
	}


	/**
	 * Set guest tracking key
	 * 
	 * @param string $key
	 */
	public function set_key( $key ) {
		$this->set_prop( 'tracking_key', ES_Clean::string( $key ) );
	}

	/**
	 * Get guest tracking key
	 * 
	 * @deprecated Use customer key
	 *
	 * @return string
	 */
	public function get_key() {
		return ES_Clean::string( $this->get_prop( 'tracking_key' ) );
	}

	/**
	 * Get a single meta value by key.
	 *
	 * Returns an empty string if field is empty or doesn't exist.
	 *
	 * @param string $key
	 *
	 * @return mixed
	 */
	public function get_meta( $meta_key = '' ) {
		return ! empty( $this->data['meta'][ $meta_key ] ) ? $this->data['meta'][ $meta_key ] : '';
	}

	/**
	 * Set guest creation date
	 * 
	 * @param DateTime $date
	 */
	public function set_date_created( $date ) {
		$this->set_date_column( 'created', $date );
	}

	/**
	 * Get guest creation date
	 * 
	 * @return bool|DateTime
	 */
	public function get_date_created() {
		return $this->get_date_column( 'created' );
	}


	/**
	 * Get guest full name
	 * 
	 * @return string
	 */
	public function get_full_name() {
		/* translators: 1. Guest first name 2. Guest last name */
		return trim( sprintf( _x( '%1$s %2$s', 'full name', 'email-subscribers' ), $this->get_first_name(), $this->get_last_name() ) );
	}

	/**
	 * Get guest first name
	 * 
	 * @return string
	 */
	public function get_first_name() {
		return $this->get_checkout_field( 'billing_first_name' );
	}


	/**
	 * Get guest last name
	 * 
	 * @return string
	 */
	public function get_last_name() {
		return $this->get_checkout_field( 'billing_last_name' );
	}


	/**
	 * Get guest phone
	 * 
	 * @return string
	 */
	public function get_phone() {
		return $this->get_checkout_field( 'billing_phone' );
	}


	/**
	 * Get guest country
	 * 
	 * @return string
	 */
	public function get_country() {
		return $this->get_checkout_field( 'billing_country' );
	}


	/**
	 * Get guest company
	 * 
	 * @return string
	 */
	public function get_company() {
		return $this->get_checkout_field( 'billing_company' );
	}


	/**
	 * Get guest address' first line
	 * 
	 * @return string
	 */
	public function get_address_1() {
		return $this->get_checkout_field( 'billing_address_1' );
	}


	/**
	 * Get guest address' second line
	 * 
	 * @return string
	 */
	public function get_address_2() {
		return $this->get_checkout_field( 'billing_address_2' );
	}


	/**
	 * Get guest city
	 * 
	 * @return string
	 */
	public function get_city() {
		return $this->get_checkout_field( 'billing_city' );
	}


	/**
	 * Get guest state
	 * 
	 * @return string
	 */
	public function get_state() {
		return $this->get_checkout_field( 'billing_state' );
	}


	/**
	 * Get guest postcode
	 * 
	 * @return string
	 */
	public function get_postcode() {
		return $this->get_checkout_field( 'billing_postcode' );
	}

	/**
	 * Retrieve a valid checkout field if one is stored in meta or get from an order belonging to the guest
	 *
	 * @param $field
	 * 
	 * Set true to bypass most recent order
	 * @return mixed
	 */
	public function get_checkout_field( $field ) {

		if ( ! IG_ES_WC_Carts::is_checkout_capture_field( $field ) ) {
			return false;
		}

		return $this->get_meta( $field );
	}

	/**
	 * Get cart
	 * 
	 * @return IG_ES_Cart|false
	 */
	public function get_cart() {
		return IG_ES_Cart_Factory::get_by_guest_id( $this->get_id() );
	}
	
	/**
	 * The locked guest status is used by the presubmit capture module.
	 *
	 * If a guest is not locked their email address may be changed if the capture email address is modified.
	 * But allowing emails to change later can cause for confusing issues, hence this logic.
	 *
	 * A guest becomes locked 10 minutes after they are created or once they place an order.
	 *
	 * @return bool
	 */
	public function is_locked() {
		$created = $this->get_date_created();
		if ( ! $created ) {
			return true; // guest has no created date so default to locked
		}

		$locking_timestamp = time() - (int) apply_filters( 'ig_es_guest_locking_timeout', 600, $this ); // defaults to 10 minutes

		if ( $created->getTimestamp() < $locking_timestamp ) {
			return true;
		}

		return false;
	}

	/**
	 * Delete cart
	 */
	public function delete_cart() {
		$cart = $this->get_cart();
		if ( $this->exists && $cart ) {
			$cart->delete_cart();
		}
	}

	/**
	 * Delete cart
	 * 
	 * @return void
	 */
	public function delete_guest() {
		$guest_id = $this->id;
		$this->delete( $guest_id );
	}

	/**
	 * Inserts or updates the model
	 * Only updates modified fields
	 *
	 * @return bool True on success, false on error.
	 */
	public function save() {

		if ( $this->exists ) {
			// update changed fields
			$changed_data = array_intersect_key( $this->data, array_flip( $this->changed_fields ) );

			// serialize
			$changed_data = array_map( 'maybe_serialize', $changed_data );

			if ( empty( $changed_data ) ) {
				return true;
			}

			$guest_id = $this->get_id();

			$updated = $this->update( $guest_id, $changed_data );

			if ( false === $updated ) {
				// Return here to prevent cache updates on error
				return false;
			}

			do_action( 'ig_es_object_update', $this ); // cleans object cache
		} else {
			$this->set_created( new DateTime() );
			$this->data = array_map( 'maybe_serialize', $this->data );
			
			// insert row
			$guest_id = $this->insert( $this->data );

			if ( $guest_id ) {
				$this->exists = true;
				$this->id = $guest_id;
			} else {
				/* translators: %s: Table name */
				ES()->logger->error( sprintf( __( 'Could not insert into \'%1$s\' table. \'%1$s\' may not be present in the database.', 'email-subscribers' ), $this->table_name ), $this->logger_context );

				// Return here to prevent cache updates on error
				return false;
			}
		}

		// reset changed data
		// important to reset after cache hooks
		$this->changed_fields = array();
		$this->original_data = $this->data;

		return true;
	}

	/**
	 * Set cart creation date/time
	 * 
	 * @param DateTime $date
	 */
	public function set_created( $date ) {
		$this->set_date_column( 'created', $date );
	}

	/**
	 * Set cart data option
	 * 
	 * @param $key
	 * @param $value
	 */
	public function set_prop( $key, $value ) {

		if ( is_array( $value ) && ! $value ) {
			$value = ''; // convert empty arrays to blank
		}

		$this->data[$key] = $value;
		$this->changed_fields[] = $key;
	}

	/**
	 * Method to check if cart has given data
	 * 
	 * @param $key
	 * @return bool
	 */
	public function has_prop( $key ) {
		return isset( $this->data[$key] );
	}

	/**
	 * Get cart data option
	 * 
	 * @param $key
	 * @return mixed
	 */
	public function get_prop( $key ) {
		if ( ! isset( $this->data[$key] ) ) {
			return false;
		}

		$value = $this->data[$key];
		$value = maybe_unserialize( $value );

		return $value;
	}


	/**
	 * Sets the value of a date column from  a mixed input.
	 *
	 * $value can be an instance of WC_DateTime the timezone will be ignored.
	 * If $value is a string it must be MYSQL formatted.
	 *
	 * @param string                                 $column
	 * @param DateTime|\DateTime|string $value
	 */
	protected function set_date_column( $column, $value ) {
		if ( is_a( $value, 'DateTime' ) ) {
			// convert to UTC time
			$utc_date = new DateTime();
			$utc_date->setTimestamp( $value->getTimestamp() );
			$this->set_prop( $column, $utc_date->format( 'Y-m-d H:i:s' ) );
		} elseif ( $value ) {
			$this->set_prop( $column, ES_Clean::string( $value ) );
		}
	}

	/**
	 * Get datetime of workflow queue
	 * 
	 * @param $column
	 * @return bool|DateTime
	 */
	protected function get_date_column( $column ) {
		$prop = $this->get_prop( $column );
		if ( $column && $prop ) {
			return new DateTime( $prop );
		}

		return false;
	}
}
