<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * IG_ES_WC_Carts management class
 */
class IG_ES_WC_Carts {

	/** 
	 * Flag to check if cart has been changed
	 *
	 * @var bool - when true cart has been change
	 **/
	public static $is_changed = false;

	/**
	 * True if a cart is currently being restored.
	 *
	 * @var bool
	 */
	private static $is_doing_restore = false;

	/**
	 * Loaded if abandoned cart is enabled
	 */
	public static function init() {
		/** 
		 * Class name
		 *
		 * @var $self IG_ES_WC_Carts (for IDE)
		 **/
		$self = __CLASS__;

		add_action( 'wp_enqueue_scripts', array( $self, 'maybe_enqueue_presubmit_js' ) );
		add_action( 'wp_ajax_ig_es_capture_guest_email', array( $self, 'ajax_capture_guest_email' ) );
		add_action( 'wp_ajax_nopriv_ig_es_capture_guest_email', array( $self, 'ajax_capture_guest_email' ) );
		
		add_action( 'wp_ajax_ig_es_capture_checkout_field_value', array( $self, 'ajax_capture_checkout_field_value' ) );
		add_action( 'wp_ajax_nopriv_ig_es_capture_checkout_field_value', array( $self, 'ajax_capture_checkout_field_value' ) );
		
		add_action( 'ig_es_wc_abandoned_cart_worker', array( $self, 'check_for_abandoned_carts' ) );
		add_action( 'ig_es_two_days_worker', array( $self, 'clean_stored_carts' ) );

		add_action( 'woocommerce_cart_emptied', array( $self, 'cart_emptied' ) );

		// Clear customer's cart when order status changes from cancelled, failed or pending
		add_action( 'woocommerce_order_status_changed', array( $self, 'clear_cart_on_order_status_changed' ), 10, 3 );

		add_action( 'shutdown', array( $self, 'maybe_store_cart' ) );

		// change events
		add_action( 'woocommerce_add_to_cart', array( $self, 'mark_as_changed' ) );
		add_action( 'woocommerce_applied_coupon', array( $self, 'mark_as_changed' ) );
		add_action( 'woocommerce_removed_coupon', array( $self, 'mark_as_changed' ) );
		add_action( 'woocommerce_cart_item_removed', array( $self, 'mark_as_changed' ) );
		add_action( 'woocommerce_cart_item_restored', array( $self, 'mark_as_changed' ) );
		add_action( 'woocommerce_before_cart_item_quantity_zero', array( $self, 'mark_as_changed' ) );
		add_action( 'woocommerce_after_cart_item_quantity_update', array( $self, 'mark_as_changed' ) );

		add_action( 'woocommerce_after_calculate_totals', array( $self, 'trigger_update_on_cart_and_checkout_pages' ) );

		add_action( 'ig_es_session_tracking_disabled', array( $self, 'delete_stored_session_data' ) );
	}

	public static function mark_as_changed() {
		static::$is_changed = true;
	}

	public static function trigger_update_on_cart_and_checkout_pages() {
		if (
				defined( 'WOOCOMMERCE_CART' )
				|| is_checkout()
				|| did_action( 'woocommerce_before_checkout_form' ) //  support for one page checkout plugins
		) {
			self::mark_as_changed();
		}
	}

	/**
	 * Get abandone cart timeout in minutes
	 * 
	 * @return int $cart_abandoned_timeout
	 */
	public static function get_cart_abandoned_timeout() {

		$cart_abandoned_timeout = apply_filters( 'ig_es_wc_abandoned_cart_timeout', 10 ); // mins

		return $cart_abandoned_timeout;
	}

	/**
	 * Check if any active carts have been abandoned, runs every 2 minutes
	 */
	public static function check_for_abandoned_carts() {

		$cart_abandoned_timeout = self::get_cart_abandoned_timeout();

		$timeout_date = gmdate( 'Y-m-d H:i:s', strtotime( "-$cart_abandoned_timeout minutes" ) );

		$query = new IG_ES_WC_Cart_Query();
		$query->where_status( 'active' );
		$query->where_date_modified( $timeout_date );
		$query->set_limit( 100 );
		$query->set_return( 'ids' );

		$carts = $query->get_results();
		
		if ( ! $carts ) {
			return;
		}

		foreach ( $carts as $cart_data ) {
			$cart_id = $cart_data['id'];
			$cart = IG_ES_Cart_Factory::get( ES_Clean::id( $cart_id ) );
			$cart->update_status( 'abandoned' );
		}
	}


	/**
	 * Logic to determine whether we should save the cart on certain hooks
	 */
	public static function maybe_store_cart() {
		
		if ( ! self::$is_changed ) { 
			return;  // cart has not changed
		}
		if ( did_action( 'wp_logout' ) ) {
			return; // don't clear the cart after logout
		}
		
		if ( is_admin() ) {
			return;
		}
		
		// session only loaded on front end
		if ( WC()->session ) {
			$last_checkout = WC()->session->get('ig_es_checkout_processed_time');
			
			// ensure checkout has not been processed in the last 5 minutes
			// this is a fallback for a rare case when the cart session is not cleared after checkout
			if ( $last_checkout && $last_checkout > ( time() - 5 * MINUTE_IN_SECONDS ) ) {
				return;
			}
		}
		
		$customer = IG_ES_WC_Session_Tracker::get_session_customer();
		if ( $customer ) {
			self::update_stored_customer_cart( $customer );
		}
	}


	/**
	 * Updates the stored cart for a customer.
	 * Will also clear a cart if necessary.
	 *
	 * @param IG_ES_Customer $customer
	 */
	public static function update_stored_customer_cart( $customer ) {
		if ( ! $customer ) {
			return;
		}

		// If the customer is registered and is logged out, their cart will be emptied
		// At this point we are tracking them via cookie so it doesn't make sense to clear their stored cart
		if ( $customer->is_registered() && ! is_user_logged_in() && WC()->cart->is_empty() ) {
			return;
		}

		$cart = $customer->get_cart();
		if ( $cart ) {
			// delete cart if empty otherwise update it
			if ( WC()->cart->is_empty() ) {
				$cart->delete_cart();
			} else {
				$cart->sync();
			}
		} else {
			// create a new cart if the current session cart isn't empty
			if ( ! WC()->cart->is_empty() ) {
				$cart = new IG_ES_Cart();
				if ( $customer->is_registered() ) {
					$cart->set_user_id( $customer->get_user_id() );
				} else {
					$cart->set_guest_id( $customer->get_guest_id() );
				}
				$cart->set_token();
				$cart->sync();
			}
		}
	}


	/**
	 * The woocommerce_cart_emptied fires when an order is placed and the cart is emptied.
	 * It does NOT fire when a user empties their cart.
	 * It appears to also NOT fire when an a pending or failed order is generated,
	 * important that it remains this way for the abandoned_cart_includes_pending_orders option
	 */
	public static function cart_emptied() {
		if ( did_action( 'wp_logout' ) ) {
			return; // don't clear cart after logout
		}

		// Ensure carts are cleared for users registered at checkout
		$user_id = IG_ES_WC_Session_Tracker::get_detected_user_id();

		if ( $user_id ) {
			$cart = IG_ES_Cart_Factory::get_by_user_id( $user_id );
			if ( $cart ) {
				$cart->delete_cart();
			}
		} else {
			$guest = IG_ES_WC_Session_Tracker::get_current_guest();
			if ( $guest ) {
				$guest->delete_cart();
			}
		}

		self::$is_changed = false; // cart is up-to-date
	}


	/**
	 * Clear cart when transition changes from pending, cancelled or failed
	 *
	 * @param $order_id
	 * @param $old_status
	 * @param $new_status
	 */
	public static function clear_cart_on_order_status_changed( $order_id, $old_status, $new_status ) {
		$failed_statuses = array( 'pending', 'failed', 'cancelled' );

		if ( in_array( $old_status, $failed_statuses ) && ! in_array( $new_status, $failed_statuses ) ) {
			self::clear_cart_by_order( $order_id );
		}
	}


	/**
	 * Clears and carts that match the customer from an order
	 *
	 * @param $order_id
	 */
	public static function clear_cart_by_order( $order_id ) {
		$order = wc_get_order( ES_Clean::id( $order_id ) );
		if ( ! $order ) {
			return;
		}

		$user_id = $order->get_user_id();
		if ( $user_id ) {
			$cart = IG_ES_Cart_Factory::get_by_user_id( $user_id );
			if ( $cart ) {
				$cart->delete_cart();
			}
		} else {
			// clear by email
			$guest = IG_ES_Guest_Factory::get_by_email( ES_Clean::email( $order->get_billing_email() ) );
			if ( $guest ) {
				$guest->delete_cart();
			}
		}

		self::$is_changed = false; // cart is up-to-date
	}


	/**
	 * Restores a cart into the current session.
	 *
	 * @param IG_ES_Cart|bool $cart
	 *
	 * @return bool
	 */
	public static function restore_cart( $cart ) {
		if ( ! $cart || ! $cart->has_items() ) {
			return false;
		}

		self::$is_doing_restore = true;

		$notices_backup = wc_get_notices();

		// merge restored items with existing
		$existing_items = WC()->cart->get_cart_for_session();

		foreach ( $cart->get_items() as $item ) {
			if ( isset( $existing_items[ $item->get_key() ] ) ) {
				continue; // item already exists in cart
			}

			WC()->cart->add_to_cart( $item->get_product_id(), $item->get_quantity(), $item->get_variation_id(), $item->get_variation_data(), $item->get_data() );
		}

		// restore coupons
		foreach ( $cart->get_coupons() as $coupon_code => $coupon_data ) {
			if ( ! WC()->cart->has_discount( $coupon_code ) ) {
				WC()->cart->add_discount( $coupon_code );
			}
		}

		// clear notices for when a added coupons or products is added to cart
		WC()->session->set( 'wc_notices', $notices_backup );

		self::$is_doing_restore = false;

		do_action( 'ig_es/cart/restored', $cart );

		return true;
	}


	/**
	 * Is a cart restore in progress?
	 *
	 * @since 4.4.0
	 *
	 * @return bool
	 */
	public static function is_doing_restore() {
		return self::$is_doing_restore;
	}


	/**
	 * Delete old inactive carts
	 */
	public static function clean_stored_carts() {
		global $wpdb;

		$clear_inactive_carts_after = apply_filter( 'ig_es_clear_inactive_carts_after', 60 );

		$delay_date = new DateTime();
		$delay_date->modify("-$clear_inactive_carts_after days");

		$wpdb->query( $wpdb->prepare("
			DELETE FROM `{$wpdb->prefix}ig_wc_cart`
			WHERE last_modified < %s",
			$delay_date->to_mysql_string()
		));
	}

	/**
	 * Enqueue JS to capture WooCommerce checkout data before submitting checkout form
	 */
	public static function maybe_enqueue_presubmit_js() {
		
		if ( is_user_logged_in() || ! is_checkout() || ! IG_ES_WC_Session_Tracker::session_tracking_enabled() ) {
			return;
		}

		$asset_url = untrailingslashit( plugins_url( '/', __FILE__ ) ) . '/assets';

		wp_register_script( 'ig-es-pre-submit', $asset_url . '/js/pre-submit.js', array( 'jquery' ), ES_PLUGIN_VERSION, true );
		wp_localize_script( 'ig-es-pre-submit', 'ig_es_presubmit_params', self::get_js_params() );

		wp_enqueue_script( 'ig-es-pre-submit' );
	}

	/**
	 * Get required data for js
	 * 
	 * @return array
	 */
	public static function get_js_params() {

		$params = array();
		$guest  = IG_ES_WC_Session_Tracker::get_current_guest();

		$params['guest_id']                     = $guest ? $guest->get_id(): 0;
		$params['email_capture_selectors'] 		= self::get_email_capture_selectors();
		$params['checkout_capture_selectors'] 	= self::get_checkout_capture_fields();
		$params['ajax_url'] 					= admin_url( 'admin-ajax.php' );
		$params['security'] 					= wp_create_nonce( 'ig-es-pre-submit-nonce' );

		return $params;
	}

	/**
	 * Get selectors for email field
	 * 
	 * @return array
	 */
	public static function get_email_capture_selectors() {
		return apply_filters( 'ig_es_guest_capture_fields', array(
			'.woocommerce-checkout [type="email"]',
			'#billing_email',
		));
	}

	/**
	 * Check if given field is a checkout field
	 * 
	 * @param $field_name
	 * @return bool
	 */
	public static function is_checkout_capture_field( $field_name ) {
		return in_array( $field_name, self::get_checkout_capture_fields() );
	}

	/**
	 * Get checkout field list
	 * 
	 * @return array
	 */
	public static function get_checkout_capture_fields() {
		return apply_filters( 'ig_es_checkout_capture_fields', array(
			'billing_first_name',
			'billing_last_name',
			'billing_company',
			'billing_phone',
			'billing_country',
			'billing_address_1',
			'billing_address_2',
			'billing_city',
			'billing_state',
			'billing_postcode'
		));
	}

	/**
	 * Capture guest email
	 */
	public static function ajax_capture_guest_email() {

		check_ajax_referer( 'ig-es-pre-submit-nonce', 'security' );

		$email = ES_Clean::email( ig_es_get_request_data('email') );

		$customer = IG_ES_WC_Session_Tracker::set_session_by_captured_email( $email );

		if ( ! $customer ) {
			wp_send_json_error();
		}

		$guest = $customer->get_guest();

		if ( ! $guest ) {
			wp_send_json_error();
		}

		$guest_id = $guest->get_id();
		if ( ! empty( $guest_id ) ) {
			$checkout_fields = ES_Clean::recursive( ig_es_get_request_data( 'checkout_fields', array(), false ) );
			if ( ! empty( $checkout_fields ) ) {
				$meta_data = array();
				foreach ( $checkout_fields as $field_name => $field_value ) {
	
					if ( ! self::is_checkout_capture_field( $field_name ) || empty( $field_value ) ) {
					  continue; // IMPORTANT don't save the field if it is empty
					}
	
					$meta_data[ $field_name ] = stripslashes( $field_value );
				}
				if ( ! empty( $meta_data ) ) {
					$guest->update_meta( $guest_id, $meta_data );
				}
			} else {
				$location = wc_get_customer_default_location();
				if ( $location['country'] ) {
					$guest->update_meta( 'billing_country', $location['country'] );
				}
			}
		}

		wp_send_json_success(array(
		   'guest_id' => $guest_id,
		));
	}

	/**
	 * Capture an additional field from the checkout page
	 */
	public static function ajax_capture_checkout_field_value() {

		check_ajax_referer( 'ig-es-pre-submit-nonce', 'security' );

		if ( is_user_logged_in() || ! IG_ES_WC_Session_Tracker::session_tracking_enabled() ) {
			return;
		}

		$guest_id    = absint( ig_es_get_request_data( 'guest_id' ) );
		$field_name  = ES_Clean::string( ig_es_get_request_data( 'field_name' ) );
		$field_value = stripslashes( ES_Clean::string( ig_es_get_request_data( 'field_value' ) ) );

		$guest = IG_ES_WC_Session_Tracker::get_current_guest();

		if ( ! $guest || $guest_id != $guest->get_id() ) {
			wp_send_json_error();
		}

		if ( self::is_checkout_capture_field( $field_name ) ) {
			$guest->update_meta( $guest_id, array( $field_name => $field_value ) );
		}

		wp_send_json_success();
	}

	/**
	 * Delete any stored session data when user opt out
	 */
	public static function delete_stored_session_data() {

		global $woocommerce;

		// Can't look up the customer in this situation.
		if ( ! isset( $woocommerce->session ) ) {
			return;
		}

		$tracking_key = $woocommerce->session->get_customer_id();
		if ( $tracking_key ) {
			$guest = IG_ES_Guest_Factory::get_by_key( $tracking_key );
			if ( $guest instanceof IG_ES_Guest ) {
				$guest->delete_cart();
				$guest->delete_guest();
				wp_send_json_success();
			}
		}
		wp_send_json_error();
	}
}
