<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists('ES_Engagement_Score') ) {

	/**
	 * Calculate Engagement Score
	 * 
	 * @since 4.8.3
	 */
	class ES_Engagement_Score {

		/**
		 * Constructor
		 */
		public function __construct() {

			add_filter( 'manage_es_subscribers_columns', array( &$this, 'add_engagement_score_column_header' ), 99, 1 );
			add_filter( 'es_subscribers_col_data', array( $this, 'add_engagement_score_column_data' ), 11, 3 );

			add_action( 'ig_es_calculate_engagement_score', array( $this,'check_engagement_score' ) );

			add_action( 'ig_es_contact_subscribed', array( $this,'update_subscribed_contact_score' ) );	
			add_action( 'ig_es_contact_unsubscribe', array( $this,'update_unsubscribed_contact_score' ), 10, 4 );

			add_action( 'ig_es_calculate_existing_subscribers_engagement_score', array( $this, 'calculate_existing_subscribers_engagement_score' ) );
		}

		/**
		 * Add engagement score column header
		 * 
		 * @param array $subscribers_columns
		 * 
		 * @return array $subscribers_columns
		 */
		public function add_engagement_score_column_header( $subscribers_columns ) {

			$score_col = array(
				'engagement_score' => __( 'Score', 'email-subscribers' ),
			);

			$subscribers_columns = ig_es_array_insert_after( $subscribers_columns, 'subscriber' , $score_col );

			return $subscribers_columns;
		}

		/**
		 * Add engagement score column on audience screen
		 * 
		 * @param array $data contact data
		 * @param string $column_name Column name
		 * @param ES_Contacts_Table $contacts_list_class Contact table class
		 */
		public function add_engagement_score_column_data( $contact_data, $column_name, $contacts_list_class ) {

			if ( 'engagement_score' === $column_name ) {
				$score_class = '';
				$score_text  = '';
				if ( is_numeric( $contact_data['engagement_score'] ) ) {
					$score_class = 'bad';
					if ( $contact_data['engagement_score'] > 0 ) {
						$score_text = number_format_i18n( $contact_data['engagement_score'], 1 );
					} else {
						$score_text = 0;
					}
					if ( $contact_data['engagement_score'] >= 4 ) {
						$score_class = 'excellent';
					} elseif ( $contact_data['engagement_score'] >= 3 ) {
						$score_class = 'good';
					} elseif ( $contact_data['engagement_score'] >= 2 ) {
						$score_class = 'low';
					} elseif ( $contact_data['engagement_score'] >= 1 ) {
						$score_class = 'very-low';
					}
				}
				$contact_data['engagement_score'] = ( is_numeric( $contact_data['engagement_score'] ) ? '<div class="rating ' . $score_class . '">' . $score_text . '</div>' : '-' );
			}

			return $contact_data;
		}

		/**
		 * Check engagement score of subscribers who have opened/click email after last cron run
		 */
		public function check_engagement_score() {

			global $wpdb;

			$last_run_time = get_option( 'ig_es_engagement_score_cron_last_run_at', 0 );

			if ( empty( $last_run_time ) ) {
				$last_run_time = strtotime( 'today midnight' );
			}
			
			$engaged_subscribers_ids = $wpdb->get_col(
				$wpdb->prepare(
					"SELECT DISTINCT(`contact_id`) AS `contact_id` FROM {$wpdb->prefix}ig_actions WHERE `campaign_id` != 0 AND type IN( %d, %d, %d ) AND `updated_at` > %d;",
					IG_MESSAGE_SENT,
					IG_MESSAGE_OPEN,
					IG_LINK_CLICK,
					$last_run_time
				)
			);

			if ( ! empty( $engaged_subscribers_ids ) ) {
				$subscriber_batches = array_chunk( $engaged_subscribers_ids, 1000 );
				foreach ( $subscriber_batches as $batch ) {
					$this->update_engagement_score( $batch );
				}
			}

			update_option( 'ig_es_engagement_score_cron_last_run_at', time(), false );
		}

		/**
		 * Update subscribed contacts score
		 * 
		 * @param array $contact_data contact data
		 */
		public function update_subscribed_contact_score( $contact_data ) {
			$contact_id = ! empty( $contact_data['contact_id'] ) ? $contact_data['contact_id'] : 0;
			if ( ! empty( $contact_id ) ) {
				$this->update_engagement_score( array( $contact_id ), IG_CONTACT_SUBSCRIBE );
			}
		}

		/**
		 * Update unsubscribed contacts score
		 * 
		 * @param int $contact_id contact id
		 * @param array $list_ids contact's list id
		 */
		public function update_unsubscribed_contact_score( $contact_id, $message_id, $campaign_id, $unsubscribe_lists ) {
			$this->update_engagement_score( array( $contact_id ), IG_CONTACT_UNSUBSCRIBE );
		}

		/**
		 * Update engagement score of passed subscribed IDs
		 * 
		 * @param array $subscriber_ids Subscriber IDs
		 * @param int $event_type event type
		 */
		public function update_engagement_score( $subscriber_ids = array(), $event_type = IG_MESSAGE_OPEN ) {

			global $wpbd;

			$subscriber_ids_count        = count( $subscriber_ids );
			$subscriber_ids_placeholders = array_fill( 0, $subscriber_ids_count, '%d' );

			

			if ( in_array( $event_type, array( IG_MESSAGE_OPEN, IG_LINK_CLICK ), true ) ) {

				// We consider only one year old data for calculating the engagement score.
				$start_time = strtotime( '-1 year', time() );

				$subscribe_score = IG_ES_SUBSCRIBE_SCORE;
				$open_score      = IG_ES_OPEN_SCORE;
				$click_score     = IG_ES_CLICK_SCORE;
				$query_args      = array( $subscribe_score, $open_score, $click_score, $start_time );
				$query_args      = array_merge( $query_args, $subscriber_ids );
				
				$updated = $wpbd->query(
					$wpbd->prepare(
						"UPDATE {$wpbd->prefix}ig_contacts AS contacts_table
						LEFT JOIN (
							SELECT 
							`contact_id`,
							IF( SUM(`sent`) = 0, %d, ( ( ( SUM( `opened` ) * %d + SUM(`clicked`) * %d ) ) / ( SUM(`sent`) * 5 ) ) * 5 ) AS 'score'
							FROM ( SELECT
								contact_id,
								IF( `message_id` != 0, `message_id`, `campaign_id` ) AS 'campaign_id',
								MAX( IF( `type` = 2, 1, 0 ) ) AS 'sent',
								MAX( IF( `type` = 3, 1, 0 ) ) AS 'opened',
								MAX( IF( `type` = 4, 1, 0 ) ) AS 'clicked'
								FROM `{$wpbd->prefix}ig_actions` AS actions_table WHERE ( campaign_id != 0 AND campaign_id IS NOT NULL ) AND `updated_at` > %d AND contact_id IN(
									" . implode( ',', $subscriber_ids_placeholders ) . '
								) GROUP BY `actions_table`.`contact_id`, `actions_table`.`campaign_id`, `actions_table`.`message_id` ORDER BY `actions_table`.`contact_id` ) AS campaigns_data GROUP BY contact_id )
						AS score_table
						ON `contacts_table`.`id` = `score_table`.`contact_id`
						SET `contacts_table`.`engagement_score` = `score_table`.`score`
						WHERE `contacts_table`.`id` = `score_table`.`contact_id` AND `contacts_table`.`unsubscribed` != 1;',
						$query_args
					)
				);
				
			} elseif ( in_array( $event_type, array( IG_CONTACT_SUBSCRIBE, IG_CONTACT_UNSUBSCRIBE ), true ) ) {
				if ( IG_CONTACT_SUBSCRIBE === $event_type ) {
					$engagement_score = IG_ES_SUBSCRIBE_SCORE;
				} elseif ( IG_CONTACT_UNSUBSCRIBE === $event_type ) {
					$engagement_score = IG_ES_UNSUBSCRIBE_SCORE;
				}

				$query_args = array( $engagement_score );
				$query_args = array_merge( $query_args, $subscriber_ids );

				$wpbd->query(
					$wpbd->prepare(
						"UPDATE {$wpbd->prefix}ig_contacts
						SET `engagement_score` = %f
						WHERE `id` = " . implode( ',', $subscriber_ids_placeholders ) . '',
						$query_args
					)
				);
			}
		}

		/**
		 * Calculate engagement score of existing subscribers
		 */
		public function calculate_existing_subscribers_engagement_score() {

			global $wpdb;
			$batch_start_time = time();

			$subscriber_ids = $wpdb->get_col( "SELECT id FROM {$wpdb->prefix}ig_contacts WHERE unsubscribed != 1 AND engagement_score IS NULL LIMIT 0, 1000" );

			if ( ! empty( $subscriber_ids ) ) {
				$this->update_engagement_score( $subscriber_ids );
			}

			// Check if there are still subscribers whos engagement score needs to be calculated
			$subscriber_ids = $wpdb->get_col( "SELECT id FROM {$wpdb->prefix}ig_contacts WHERE unsubscribed != 1 AND engagement_score IS NULL LIMIT 0, 1" );
			
			if ( ! empty( $subscriber_ids ) ) {
				$next_task_time = time() + MINUTE_IN_SECONDS; // Schedule next task after 1 minute from current time.
				IG_ES_Background_Process_Helper::add_action_scheduler_task( 'ig_es_calculate_existing_subscribers_engagement_score', array(), false, false, $next_task_time );
			}
		}

	}

	new ES_Engagement_Score();
}

