<?php
// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class ES_Subscription_Summary_Automation {
	protected $cron_hook = 'ig_es_summary_automation';

	/**
	 * Class constructor
	 */
	public function __construct() {
		add_action( $this->cron_hook, array( $this, 'handle_automation' ) );
		add_action( 'ig_es_plugin_activate', array( $this, 'schedule_summary_automation' ) );
		add_action( 'ig_es_after_settings_save', array( $this, 'ig_es_after_settings_save' ) );
		add_action( 'ig_es_plugin_deactivate', array( $this, 'clear_scheduled_automation' ) );
	}

	/**
	 * Handle automation while saving the settings.This action may clear or reschedule the automation as per need.
	 *
	 * @param $settings
	 */
	public function ig_es_after_settings_save( $settings ) {
		$is_automation_enabled = isset( $settings['ig_es_enable_summary_automation'] ) ? $settings['ig_es_enable_summary_automation'] : 'yes';
		if ( 'yes' === $is_automation_enabled ) {
			$this->schedule_summary_automation( true, false );
		} else {
			$this->clear_scheduled_automation();
		}
	}

	/**
	 * Get the HTML content required for email
	 *
	 * @param array $data
	 *
	 * @return false|string
	 */
	public function get_mail_content( $data = array() ) {
		$file_path = ES_PLUGIN_DIR . 'pro/templates/email/es-admin-subscription-summary.php';
		if ( file_exists( $file_path ) ) {
			ob_start();
			extract( $data );
			require_once $file_path;

			return ob_get_clean();
		}

		return '';
	}

	/**
	 * Get all the summary that needed for the automation
	 *
	 * @param $interval numeric Data required in particular interval in days
	 *
	 * @return array
	 */
	public function get_automation_summary( $interval = 7 ) {
		$today = time();

		return array(
			'contacts_growth'     => ES_Reports_Data::get_contacts_growth_percentage( $interval ),
			'total_subscribed'    => ES_Reports_Data::get_total_subscribed_contacts( $interval ),
			'total_unsubscribed'  => ES_Reports_Data::get_total_unsubscribed_contacts( $interval ),
			'total_sent_mails'    => ES_Reports_Data::get_total_emails_sent( $interval ),
			'total_opened_mails'  => ES_Reports_Data::get_total_contacts_opened_emails( $interval ),
			'total_clicked_mails' => ES_Reports_Data::get_total_contacts_clicks_links( $interval ),
			'start_date'          => gmdate( 'F d', $today - ( $interval * DAY_IN_SECONDS ) ),
			'end_date'            => gmdate( 'F d, Y', $today ),
		);
	}

	/**
	 * Handle the cron event
	 *
	 * @return bool
	 */
	public function handle_automation() {
		$admin_email = get_option( 'admin_email' );
		if ( is_email( $admin_email ) ) {
			$user       = get_user_by( 'email', $admin_email );
			$admin_name = '';
			if ( $user instanceof WP_User ) {
				$admin_name = $user->display_name;
			}
			$summary    = $this->get_automation_summary();
			$extra_data = array(
				'site_name'     => get_option( 'blogname' ),
				'admin_name'    => $admin_name,
				'logo_url'      => ES_PLUGIN_URL . 'lite/admin/images/es-logo-64x64.png',
				'dashboard_url' => admin_url( 'admin.php?page=es_dashboard' ),
				'reports_url'   => admin_url( 'admin.php?page=es_reports' ),
			);
			$summary    = array_merge( $summary, $extra_data );
			$html       = $this->get_mail_content( $summary );
			$subject    = __('Weekly Report from Email Subscribers', 'email-subscribers');

			ES()->mailer->add_tracking_pixel   = false;
			ES()->mailer->add_unsubscribe_link = false;

			return ES()->mailer->send( $subject, $html, $admin_email );
		}

		return false;
	}

	/**
	 * Handles scheduling the automation
	 *
	 * @param false $force_clear Do I need to clear the scheduled event before scheduling the event
	 */
	public function schedule_summary_automation( $force_clear = false, $check_setting_before_schedule = true ) {
		//Don't give workload to site users
		if ( ! is_admin() ) {
			return false;
		}
		if ( $check_setting_before_schedule ) {
			$ig_es_enable_summary_automation = get_option( 'ig_es_enable_summary_automation', 'yes' );
			if ( 'no' == $ig_es_enable_summary_automation ) {
				return false;
			}
		}
		$run_cron_on   = get_option( 'ig_es_run_cron_on', 'monday' );
		$run_cron_time = get_option( 'ig_es_run_cron_time', '4pm' );
		$day_and_time  = "{$run_cron_on} {$run_cron_time}";
		//Clear the scheduled hook,
		//This will help , when we want to clear all scheduled hook, and then reschedule.
		if ( $force_clear ) {
			$this->clear_scheduled_automation();
		}
		//Schedule an action if it's not already scheduled
		if ( ! wp_next_scheduled( $this->cron_hook ) ) {
			$scheduled_time = $this->get_schedule_time( $day_and_time );
			if ( ! is_null( $scheduled_time ) ) {
				$response = wp_schedule_event( $scheduled_time, 'weekly', $this->cron_hook, array(), true );
				if ( $response instanceof WP_Error ) {
					//wp_die( $response->get_error_message() );
					return false;
				}
			}
		}

		return true;
	}

	/**
	 * Get the schedule time from the day
	 *
	 * @param $day_and_time string
	 *
	 * @return false|float|int
	 */
	public function get_schedule_time( $day_and_time ) {
		try {
			$date                   = new DateTime( $day_and_time );
			$scheduled_datetime     = $date->format( 'Y-m-d h:i:s A' );
			$scheduled_datetime_gmt = get_gmt_from_date( $scheduled_datetime );

			return strtotime( $scheduled_datetime_gmt );
		} catch ( Exception $e ) {
			return null;
		}
	}

	/**
	 * Clear the scheduled automation
	 */
	public function clear_scheduled_automation() {
		wp_clear_scheduled_hook( $this->cron_hook );
	}
}

new ES_Subscription_Summary_Automation();
