<?php
// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'IG_ES_Unsubscribe_Feedback' ) ) {

	/**
	 * Class IG_ES_Unsubscribe_Feedback
	 *
	 * @since 4.6.8
	 */
	class IG_ES_Unsubscribe_Feedback {

		/**
		 * IG_ES_DB_Unsubscribe_Feedback object
		 *
		 * @var $db
		 */
		protected $db;

		/**
		 * IG_ES_Unsubscribe_Feedback constructor.
		 *
		 */
		public function __construct() {

			$this->db = new IG_ES_DB_Unsubscribe_Feedback();

			add_action( 'init', array( $this, 'init' ) );
		}

		/**
		 * Init Actions
		 */
		public function init() {

			// Action to show unsubscribe feedback field on the unsubscribe page
			add_action( 'ig_es_unsubscribe_form_before_end', array( $this, 'show_unsubscribe_feedback_field' ) );
			
			// Save unsubscribe field on the unsubscribe page
			add_action( 'ig_es_contact_unsubscribe', array( $this, 'save_unsubscribe_feedback' ), 10, 4 );
		}

		/**
		 * Get default unsubscribe feedbacks.
		 * 
		 * @return array $unsubscribe_feedbacks
		 */
		public function get_default_unsubscribe_feedbacks() {

			$unsubscribe_feedbacks = array(
				'emails_not_relevant' => __( 'Your emails are not relevant to me', 'email-subscribers' ),
				'emails_too_frequent' => __( 'Your emails are too frequent', 'email-subscribers' ),
				'unknown_sign_up'     => __( 'I don\'t remember signing up for this', 'email-subscribers' ),
				'not_interested'      => __( 'I no longer want to receive these emails', 'email-subscribers' ),
				'spam_emails'         => __( 'The emails are spam and should be reported', 'email-subscribers' ),
			);

			$unsubscribe_feedbacks = apply_filters( 'ig_es_default_unsubscribe_feedbacks', $unsubscribe_feedbacks );

			return $unsubscribe_feedbacks;
		}
		
		/**
		 * Show unsubscribe feedback field
		 */
		public function show_unsubscribe_feedback_field() {
			$unsubscribe_feedbacks = $this->get_default_unsubscribe_feedbacks();
			if ( ! empty( $unsubscribe_feedbacks ) ) {
				?>
				<div class="ig_es_unsubscribe_feedback_section p-3 border-t border-gray-150">
					<div class="ig_es_form_heading ">
						<p class="text-base tracking-wide text-gray-700 font-medium">
							<?php echo esc_html__( 'We would like to know why you are unsubscribing', 'email-subscribers' ); ?>
						</p>
					</div>
					<div class="ig_es_feedback_slug_wrapper mb-3">
					<select name="feedback_slug" onchange="toggle_feedback_textarea(this.value);" class="block w-3/4 h-8 text-sm rounded-md cursor-pointer h-9 form-select">
						<option value="">
							<?php
							echo esc_html__( 'Please select reason', 'email-subscribers' );
							?>
						</option>
						<?php
						foreach ( $unsubscribe_feedbacks as $unsubscribe_feedback_slug => $unsubscribe_feedback_text ) {
							?>
						<option value="<?php echo esc_attr( $unsubscribe_feedback_slug ); ?>">
							<?php
								echo esc_html( $unsubscribe_feedback_text );
							?>
						</option>
						<?php
						}
						?>
						<option value="other">
						<?php
							echo esc_html__( 'Other', 'email-subscribers' );
						?>
						</option>
					</select>
					</div>
					<div class="ig_es_feedback_text_wrapper">
					<textarea id="feedback_text" name="feedback_text" class="mt-1 w-3/4 h-20 border border-gray-300 rounded-md form-textarea" style="display:none" placeholder="<?php echo esc_attr__( 'Please share the reason', 'email-subscribers' ); ?>"></textarea>
					</div>
				</div>
				<script type="text/javascript">
				function toggle_feedback_textarea( value ){
					if ( 'other' === value ) {
						document.getElementById('feedback_text').style.display = 'block';
					} else {
						document.getElementById('feedback_text').style.display = 'none';
					}
				}
				</script>
				<?php
			}
		}

		/**
		 * Save unsubscribe feedback
		 *
		 * @param $contact_id
		 * @param $message_id
		 * @param $campaign_id
		 * @param array $list_ids
		 *
		 */
		public function save_unsubscribe_feedback( $contact_id, $mailing_queue_id, $campaign_id, $list_ids = array() ) {
			
			if ( is_array( $list_ids ) && count( $list_ids ) > 0 ) {

				$feedback_slug = ig_es_get_request_data( 'feedback_slug' );
				if ( ! empty( $feedback_slug ) ) {
					$default_unsubscribe_feedbacks = $this->get_default_unsubscribe_feedbacks();
					// Check if subscribe has selected a default feedback option.
					if ( isset( $default_unsubscribe_feedbacks[ $feedback_slug ] ) ) {
						$feedback_text = $default_unsubscribe_feedbacks[ $feedback_slug ];
					} else {
						$feedback_text = ig_es_get_request_data( 'feedback_text' );
					}
				} else {
					$feedback_text = '';
				}

				foreach ( $list_ids as $list_id ) {
					$existing_feedback_id = $this->db->get_existing_feedback_id( $contact_id, $list_id );
					if (  empty( $existing_feedback_id ) ) {
						$feedback_data = array(
							'contact_id'       => $contact_id,
							'list_id'          => $list_id,
							'campaign_id'      => $campaign_id,
							'mailing_queue_id' => $mailing_queue_id,
							'feedback_slug'    => $feedback_slug,
							'feedback_text'    => $feedback_text,
							'created_at'	   => ig_get_current_date_time(),
							'updated_at'	   => ig_get_current_date_time(),
						);
						$this->db->insert_feedback( $feedback_data );
					} else {
						$feedback_data = array(
							'campaign_id'      => $campaign_id,
							'mailing_queue_id' => $mailing_queue_id,
							'feedback_slug'    => $feedback_slug,
							'feedback_text'    => $feedback_text,
							'updated_at'	   => ig_get_current_date_time(),
						);
						$this->db->update_feedback( $existing_feedback_id, $feedback_data );
					}
				}
			}
		}

	}
}

new IG_ES_Unsubscribe_Feedback();
