<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Class for WooCommerce Coupon
 *
 * @since 5.3.4
 */
class IG_ES_Coupon {

	/**
	 * Search post table for shop_coupon by search keyword
	 *
	 * @param $keyword
	 * @param false $exclude_personalized
	 *
	 * @return array
	 */
	public static function get_coupons_by_keyword( $keyword, $exclude_personalized = false ) {
		if ( ! empty( $keyword ) ) {
			$args = [
				'post_type'      => 'shop_coupon',
				'posts_per_page' => 50,
				'no_found_rows'  => true,
				'meta_query'     => [],
				's'              => $keyword,
			];

			if ( $exclude_personalized ) {
				$args['meta_query'][] = [
					'key'     => '_is_ig_es_coupon',
					'compare' => 'NOT EXISTS',
				];
			}

			$query   = new \WP_Query( $args );
			$results = [];

			foreach ( $query->posts as $coupon ) {
				if ( function_exists( 'wc_format_coupon_code' ) ) {
					$code = wc_format_coupon_code( $coupon->post_title );
				} else {
					$code = apply_filters( 'woocommerce_coupon_code', $coupon->post_title );
				}
				$results[ $code ] = $code;
			}

			return $results;
		}

		return [];
	}

	/**
	 * Generate coupon code for preview purpose
	 *
	 * @param $args
	 *
	 * @return string
	 * @since 5.3.6
	 */
	public function get_coupon_code_for_preview( $args ) {
		$arguments = wp_parse_args( $args, array( 'limit' => '', 'expires' => '0', 'prefix' => '' ) );
		$prefix    = trim( $arguments['prefix'] );

		return $this->generate_unique_coupon_code( $prefix );
	}

	/**
	 * Generate new unique coupon code from the template
	 *
	 * @param $template_coupon_code
	 * @param $args
	 *
	 * @return false|WC_Coupon
	 */
	public function create_coupon_from_template( $template_coupon_code, $args ) {
		$template_coupon_id = $this->get_coupon_id_by_code( $template_coupon_code );
		if ( empty( $template_coupon_id ) ) {
			return false;
		}

		$arguments       = wp_parse_args( $args, array( 'limit' => '', 'expires' => '0', 'prefix' => '' ) );
		$prefix          = trim( $arguments['prefix'] );
		$new_coupon_code = $this->generate_unique_coupon_code( $prefix );
		if ( ! $new_coupon_code ) {
			return false;
		}

		$coupon = [
			'post_title'   => $new_coupon_code,
			'post_content' => '',
			'post_status'  => 'publish',
			'post_author'  => 1,
			'post_type'    => 'shop_coupon',
			'post_excerpt' => __( 'Generated by Email subscribers and Newsletters', 'email-subscribers' ),
		];

		wp_insert_post( $coupon );

		$new_coupon = new \WC_Coupon( $new_coupon_code );
		// copy details from template coupon to new coupon
		$template_coupon = new \WC_Coupon( $template_coupon_id );

		$new_coupon->set_discount_type( $template_coupon->get_discount_type() );
		$new_coupon->set_amount( $template_coupon->get_amount() );
		$new_coupon->set_individual_use( $template_coupon->get_individual_use() );
		$new_coupon->set_product_ids( $template_coupon->get_product_ids() );
		$new_coupon->set_excluded_product_ids( $template_coupon->get_excluded_product_ids() );
		$new_coupon->set_usage_limit_per_user( $template_coupon->get_usage_limit_per_user() );
		$new_coupon->set_limit_usage_to_x_items( $template_coupon->get_limit_usage_to_x_items() );
		$new_coupon->set_free_shipping( $template_coupon->get_free_shipping() );
		$new_coupon->set_exclude_sale_items( $template_coupon->get_exclude_sale_items() );
		$new_coupon->set_product_categories( $template_coupon->get_product_categories() );
		$new_coupon->set_excluded_product_categories( $template_coupon->get_excluded_product_categories() );
		$new_coupon->set_minimum_amount( $template_coupon->get_minimum_amount() );
		$new_coupon->set_maximum_amount( $template_coupon->get_maximum_amount() );

		$expires = intval( $arguments['expires'] );
		if ( 0 < $expires ) {
			$new_expiry_time = time() + ( $expires * DAY_IN_SECONDS );
			$new_coupon->set_date_expires( $new_expiry_time );
		}

		$usage_limit = intval( $arguments['limit'] );
		if ( 0 < $usage_limit ) {
			$new_coupon->set_usage_limit( $usage_limit );
		}

		$new_coupon->update_meta_data( '_is_ig_es_coupon', true );

		do_action( 'ig_es_create_coupon_from_template', $new_coupon, $template_coupon, $this );

		$new_coupon->save();

		return $new_coupon;
	}

	/**
	 * Generates a unique coupon code
	 *
	 * @return string
	 */
	public function generate_unique_coupon_code( $prefix = '', $suffix = '' ) {
		$length = absint( apply_filters( 'ig_es_generate_unique_coupon_code_length', 10, $this ) );
		$code   = trim( $prefix ) . $this->generate_random_code( $length ) . trim( $suffix );
		$code   = apply_filters( 'ig_es_generated_unique_coupon_code', $code, $this );

		$coupon_id = $this->get_coupon_id_by_code( $code );
		if ( ! empty( $coupon_id ) ) {
			return $this->generate_unique_coupon_code();
		}

		return $code;
	}

	/**
	 * get coupon id from the code
	 *
	 * @param string $code
	 *
	 * @return int
	 */
	public function get_coupon_id_by_code( $code ) {
		if ( empty( $code ) ) {
			return 0;
		}
		if ( function_exists( 'wc_get_coupon_id_by_code' ) ) {
			return absint( wc_get_coupon_id_by_code( $code ) );
		} else {
			return 0;
		}
	}

	/**
	 * Generates a random key string for unique coupons.
	 *
	 * Doesn't use ambiguous and vowel characters like: a e (i l 1) (o 0) u.
	 * Doesn't run any queries to check if the coupon is actually unique.
	 *
	 * @param int $length
	 *
	 * @return string
	 */
	public function generate_random_code( $length = 10 ) {
		$chars        = 'bcdfghjkmnpqrstvwxyz23456789';
		$coupon_key   = '';
		$chars_length = strlen( $chars );

		for ( $i = 0; $i < $length; $i ++ ) {
			$coupon_key .= substr( $chars, wp_rand( 0, $chars_length - 1 ), 1 );
		}

		return $coupon_key;
	}

}

