<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'ES_Amazon_SES_Mailer' ) ) {
	/**
	 * Class ES_Amazon_SES_Mailer
	 *
	 * @since 4.6.7
	 */
	class ES_Amazon_SES_Mailer extends ES_Base_Mailer {

		/**
		 * Mailer name
		 *
		 * @since 4.8.5
		 * @var
		 */
		public $name = 'Amazon SES';

		/**
		 * Mailer Slug
		 *
		 * @since 4.8.5
		 * @var
		 */
		public $slug = 'amazon_ses';
		
		/**
		 * Access key ID
		 *
		 * @var string
		 */
		public $access_key_id = '';

		/**
		 * Secret access key
		 *
		 * @var string
		 */
		public $secret_access_key = '';

		/**
		 * Closest region
		 *
		 * @var string
		 */
		public $region = '';

		/**
		 * Flag to determine whether this mailer support batch sending or not
		 * 
		 * @var boolean
		 * 
		 * @since 4.7.0
		 */
		public $support_batch_sending = true;

		/**
		 * Stores batch sending mode
		 * 
		 * @var boolean
		 * 
		 * @since 4.7.1
		 */
		public $batch_sending_mode = 'multiple';
		
		/**
		 * Batch limit
		 * 
		 * @var boolean
		 * 
		 * @since 4.7.0
		 */
		public $batch_limit = 50;

		public $ses_message = null;
		
		public $ses_request = null;

		public $template_name = '';

		/**
		 * ES_Amazon_SES_Mailer constructor.
		 *
		 * @since 4.3.2
		 */
		public function __construct() {
			parent::__construct();

			add_filter( 'ig_es_can_use_batch_api_' . $this->slug, array( $this, 'can_use_batch_api' ), 10, 3 );
		}

		/**
		 * Set mailer data
		 * 
		 * @since 4.7.0
		 */
		public function set_mailer_data() {

			// Reset body and headers.
			$this->reset_mailer_data();
			
			ES()->logger->info( 'Start Sending Email Using Amazon SES', $this->logger_context );

			$ig_es_mailer_settings = get_option( 'ig_es_mailer_settings', array() );

			if ( ES()->is_const_defined( 'Amazon_SES', 'access_key_id' ) ) {
				$this->access_key_id = ES()->get_const_value( 'Amazon_SES', 'access_key_id' );
			} else {
				$this->access_key_id = ! empty( $ig_es_mailer_settings['Amazon_SES']['access_key_id'] ) ? $ig_es_mailer_settings['Amazon_SES']['access_key_id'] : '';
			}

			if ( empty( $this->access_key_id ) ) {
				return $this->do_response( 'error', __( 'Access Key ID is empty.', 'email-subscribers' ) );
			}

			if ( ES()->is_const_defined( 'Amazon_SES', 'secret_access_key' ) ) {
				$this->secret_access_key = ES()->get_const_value( 'Amazon_SES', 'secret_access_key' );
			} else {
				$this->secret_access_key = ! empty( $ig_es_mailer_settings['Amazon_SES']['secret_access_key'] ) ? $ig_es_mailer_settings['Amazon_SES']['secret_access_key'] : '';
			}

			if ( empty( $this->secret_access_key ) ) {
				return $this->do_response( 'error', __( 'Secret Access Key is empty.', 'email-subscribers' ) );
			}

			if ( ES()->is_const_defined( 'Amazon_SES', 'region' ) ) {
				$this->region = ES()->get_const_value( 'Amazon_SES', 'region' );
			} else {
				$this->region = ! empty( $ig_es_mailer_settings['Amazon_SES']['region'] ) ? $ig_es_mailer_settings['Amazon_SES']['region'] : '';
			}

			if ( empty( $this->region ) ) {
				return $this->do_response( 'error', __( 'Closest region is not set.', 'email-subscribers' ) );
			}

			$this->ses_request = new ES_Simple_Email_Service( $this->access_key_id, $this->secret_access_key, $this->region );

			//$this->set_tracking_options();
		}

		/**
		 * Set from
		 */
		public function set_from( $email, $name = '' ) {

			if ( ! filter_var( $email, FILTER_VALIDATE_EMAIL ) ) {
				return;
			}

			if ( ! empty( $name ) ) {
				$this->ses_message->setFrom( $name . ' <' . $email . '>' );
			} else {
				$this->ses_message->setFrom( $email );
			}
		}

		/**
		 * Set reply to
		 * 
		 * @param string $email
		 * 
		 * @since 4.6.7
		 */
		public function set_reply_to( $email ) {

			if ( ! filter_var( $email, FILTER_VALIDATE_EMAIL ) ) {
				return;
			}
			
			$this->ses_message->addReplyTo( $email );
		}

		/**
		 * Set email subject.
		 *
		 * @param string $subject
		 * 
		 */
		public function set_subject( $subject ) {

			$this->ses_message->setSubject( $subject );
		}

		/**
		 * Set recipients
		 */
		public function set_recipients( $recipients ) {

			if ( empty( $recipients ) ) {
				return;
			}

			$default = array( 'to' );

			foreach ( $recipients as $kind => $emails ) {
				if (
					! in_array( $kind, $default, true ) ||
					empty( $emails ) ||
					! is_array( $emails )
				) {
					continue;
				}

				$data = array();

				foreach ( $emails as $email ) {

					if ( ! filter_var( $email, FILTER_VALIDATE_EMAIL ) ) {
						continue;
					}

					$data[] = $email;
				}

				if ( ! empty( $data ) ) {
					$this->set_body_param(
						array(
							$kind => implode( ', ', $data ),
						)
					);
				}
			}
		}

		/**
		 * Set email content
		 */
		public function set_content( $content ) {

			$this->ses_message->setMessageFromString( null, $content );
		}

		public function set_email_data( $email_data = array() ) {

			$sender_email   = ! empty( $email_data['sender_email'] ) ? $email_data['sender_email']    : '';
			$sender_name    = ! empty( $email_data['sender_name'] ) ? $email_data['sender_name']      : '';
			$reply_to_email = ! empty( $email_data['reply_to_email'] ) ? $email_data['reply_to_email']: '';
			$subject        = ! empty( $email_data['subject'] ) ? $email_data['subject']              : '';
			$content        = ! empty( $email_data['content'] ) ? $email_data['content']              : '';

			$this->ses_message = new ES_Simple_Email_Service_Message();

			$this->set_from( $sender_email, $sender_name );
			$this->set_reply_to( $reply_to_email );
			$this->set_subject( $subject );
			$this->set_content( $content );

			$template_name = sanitize_title( $subject );
			// Limit template name to 64 characters as SES api don't allow template creation if template name is more than 64 characters.
			$template_name = substr( $template_name, 0, 64 );

			$response         = $this->create_email_template( $template_name, $subject, $content );
			$template_created = empty( $response->errors ) ? true : false;
			if ( $template_created ) {
				$this->ses_message->template_name = $template_name;
			}

			return $template_created;
		}

		public function create_email_template( $template_name, $template_subject, $template_content ) {

			try {
				
				$response = $this->ses_request->createTemplate($template_name, $template_subject, $template_content);
				// We get MessageId when email is sent successfully. If it is absent then email has not sent.
				if ( empty( $response['RequestId'] ) ) {

					ES()->logger->error( 'Error in Creating Template', $this->logger_context );

					return $this->do_response( 'error' );
				}
			} catch ( Exception $e ) {
				ES()->logger->error( 'Error in Creating Template', $this->logger_context );
				ES()->logger->error( $e->getMessage(), $this->logger_context );

				return $this->do_response( 'error', $e->getMessage() );
			}

			return true;
		}

		public function delete_email_template( $template_name ) {

			try {
				$response = $this->ses_request->deleteTemplate($template_name);
				// We get RequestId when deleting email template, then template is deleted successfully. If it is absent then deleting failed.
				if ( empty( $response['RequestId'] ) ) {

					ES()->logger->error( 'Error in Deleting Template', $this->logger_context );

					return $this->do_response( 'error' );
				}
			} catch ( Exception $e ) {
				ES()->logger->error( 'Error in Deleting Template', $this->logger_context );

				return $this->do_response( 'error', $e->getMessage() );
			}

			return true;
		}

		/**
		 * Reset mailer data
		 *
		 * @return array
		 *
		 * @since 4.6.14
		 */
		public function reset_mailer_data() {

			$this->body    = array();
			$this->headers = array();
		}

		/**
		 * Send Email
		 *
		 * @param ES_Message $message
		 *
		 * @return bool|WP_Error
		 */
		public function send( ES_Message $message ) {

			ES()->logger->info( 'Start Sending Email Using Amazon SES', $this->logger_context );

			$ig_es_mailer_settings = get_option( 'ig_es_mailer_settings', array() );

			if ( ES()->is_const_defined( 'Amazon_SES', 'access_key_id' ) ) {
				$this->access_key_id = ES()->get_const_value( 'Amazon_SES', 'access_key_id' );
			} else {
				$this->access_key_id = ! empty( $ig_es_mailer_settings['Amazon_SES']['access_key_id'] ) ? $ig_es_mailer_settings['Amazon_SES']['access_key_id'] : '';
			}

			if ( empty( $this->access_key_id ) ) {
				return $this->do_response( 'error', __( 'Access Key ID is empty.', 'email-subscribers' ) );
			}

			if ( ES()->is_const_defined( 'Amazon_SES', 'secret_access_key' ) ) {
				$this->secret_access_key = ES()->get_const_value( 'Amazon_SES', 'secret_access_key' );
			} else {
				$this->secret_access_key = ! empty( $ig_es_mailer_settings['Amazon_SES']['secret_access_key'] ) ? $ig_es_mailer_settings['Amazon_SES']['secret_access_key'] : '';
			}

			if ( empty( $this->secret_access_key ) ) {
				return $this->do_response( 'error', __( 'Secret Access Key is empty.', 'email-subscribers' ) );
			}

			if ( ES()->is_const_defined( 'Amazon_SES', 'region' ) ) {
				$this->region = ES()->get_const_value( 'Amazon_SES', 'region' );
			} else {
				$this->region = ! empty( $ig_es_mailer_settings['Amazon_SES']['region'] ) ? $ig_es_mailer_settings['Amazon_SES']['region'] : '';
			}

			if ( empty( $this->region ) ) {
				return $this->do_response( 'error', __( 'Closest region is not set.', 'email-subscribers' ) );
			}

			$ses_message = new IG_ES_Simple_Email_Service_Message();
			$ses_message->addTo($message->to);
			$ses_message->setFrom( $message->from_name . ' <' . $message->from . '>' );
			$ses_message->addReplyTo( $message->reply_to_email );
			$ses_message->setSubject( $message->subject );
			$ses_message->setMessageFromString( null, $message->body );
			$ses_message->setSubjectCharset( $message->charset );
			$ses_message->setMessageCharset( $message->charset );
			$ses_message->setRecipientsCharset( $message->charset );
			

			$attachments = $message->attachments;
			if ( ! empty( $attachments ) ) {
				foreach ( $attachments as $attachment_name => $attachment_path ) {
					if ( is_file( $attachment_path ) ) {
						$ses_message->addAttachmentFromFile( $attachment_name, $attachment_path );
					}
				}
			}

			try {
				$ses = new ES_Simple_Email_Service( $this->access_key_id, $this->secret_access_key, $this->region );
				$response = $ses->SendEmail($ses_message);
				// We get MessageId when email is sent successfully. If it is absent then email has not sent.
				if ( empty( $response['MessageId'] ) ) {

					$request_id = ! empty( $response['RequestId'] ) ? $response['RequestId'] : 0;

					ES()->logger->error( '[Error in Email Sending] : ' . $message->to . ' RequestId: ' . $request_id, $this->logger_context );
	
					return $this->do_response( 'error', $request_id );
				}
			} catch ( Exception $e ) {
				ES()->logger->error( '[Error in Email Sending] : ' . $message->to . ' Error: ' . $e->getMessage(), $this->logger_context );

				return $this->do_response( 'error', $e->getMessage() );
			}

			ES()->logger->info( 'Email sent successfully using Amazon SES', $this->logger_context );

			return $this->do_response( 'success' );
		}

		/**
		 * Get AWS region names
		 * 
		 * @return array $regions AWS region names
		 */
		public static function get_regions_names() {

			$regions = array(
				IG_ES_Simple_Email_Service::AWS_US_EAST_1      => esc_html__( 'US East (N. Virginia)', 'email-subscribers' ),
				IG_ES_Simple_Email_Service::AWS_US_EAST_2      => esc_html__( 'US East (Ohio)', 'email-subscribers' ),
				IG_ES_Simple_Email_Service::AWS_US_WEST_2      => esc_html__( 'US West (Oregon)', 'email-subscribers' ),
				IG_ES_Simple_Email_Service::AWS_US_GOV_WEST_1  => esc_html__( 'US AWS GovCloud', 'email-subscribers' ),
				IG_ES_Simple_Email_Service::AWS_EU_WEST_1      => esc_html__( 'EU (Ireland)', 'email-subscribers' ),
				IG_ES_Simple_Email_Service::AWS_EU_WEST_2      => esc_html__( 'EU (London)', 'email-subscribers' ),
				IG_ES_Simple_Email_Service::AWS_EU_CENTRAL_1   => esc_html__( 'EU (Frankfurt)', 'email-subscribers' ),
				IG_ES_Simple_Email_Service::AWS_EU_SOUTH_1     => esc_html__( 'EU (Milan)', 'email-subscribers' ),
				IG_ES_Simple_Email_Service::AWS_AP_SOUTH_1     => esc_html__( 'Asia Pacific (Mumbai)', 'email-subscribers' ),
				IG_ES_Simple_Email_Service::AWS_AP_NORTHEAST_1 => esc_html__( 'Asia Pacific (Tokyo)', 'email-subscribers' ),
				IG_ES_Simple_Email_Service::AWS_AP_NORTHEAST_2 => esc_html__( 'Asia Pacific (Seoul)', 'email-subscribers' ),
				IG_ES_Simple_Email_Service::AWS_AP_SOUTHEAST_1 => esc_html__( 'Asia Pacific (Singapore)', 'email-subscribers' ),
				IG_ES_Simple_Email_Service::AWS_AP_SOUTHEAST_2 => esc_html__( 'Asia Pacific (Sydney)', 'email-subscribers' ),
				IG_ES_Simple_Email_Service::AWS_CA_CENTRAL_1   => esc_html__( 'Canada (Central)', 'email-subscribers' ),
				IG_ES_Simple_Email_Service::AWS_SA_EAST_1      => esc_html__( 'South America (São Paulo)', 'email-subscribers' ),
				IG_ES_Simple_Email_Service::AWS_AF_SOUTH_1     => esc_html__( 'Africa (Cape Town)', 'email-subscribers' ),
			);

			return $regions;
		}

		/**
		 * Add into batch
		 * 
		 * @param string $email
		 * @param array $merge_tags
		 * 
		 * @since 4.7.0
		 */
		public function add_into_batch( $email, $merge_tags = array(), $message = null ) {
			
			$name        = ig_es_get_data( $merge_tags, 'name', '' );
			$first_name  = ig_es_get_data( $merge_tags, 'first_name', '' );
			$last_name   = ig_es_get_data( $merge_tags, 'last_name', '' );
			$list_name   = ig_es_get_data( $merge_tags, 'list_name', '' );
			$hash        = ig_es_get_data( $merge_tags, 'hash', '' );
			$contact_id  = ig_es_get_data( $merge_tags, 'contact_id', 0 );
			$campaign_id = ig_es_get_data( $merge_tags, 'campaign_id', 0 );
			$message_id  = ig_es_get_data( $merge_tags, 'message_id', 0 );
			$list_ids    = ig_es_get_data( $merge_tags, 'list_ids', '' );

			$link_data = array(
				'message_id'  => $message_id,
				'campaign_id' => $campaign_id,
				'contact_id'  => $contact_id,
				'email'       => $email,
				'guid'        => $hash,
				'list_ids'	  => $list_ids,
			);
			
			$subscribe_link     = ES()->mailer->get_subscribe_link( $link_data );
			$unsubscribe_link   = ES()->mailer->get_unsubscribe_link( $link_data );
			$link_variables     = ES()->mailer->get_link_variable( $contact_id );
			$tracking_pixel_url = ES()->mailer->get_tracking_pixel_url( $link_data );

			$template_variables = array(
				'NAME'             => $name,
				'FIRSTNAME'        => $first_name,
				'LASTNAME'         => $last_name,
				'LIST'         	   => $list_name,
				'HASH'             => $hash,
				'EMAIL'            => $email,
				'contact_id'       => $contact_id,
				'CAMPAIGN_ID'      => $campaign_id,
				'MESSAGE_ID'       => $message_id,
				'LIST_IDS'         => $list_ids,
				'SUBSCRIBE-LINK'   => $subscribe_link,
				'UNSUBSCRIBE-LINK' => $unsubscribe_link,
			);

			if ( ! empty( $link_variables ) ) {
				$template_variables = array_merge( $template_variables, $link_variables );
			}

			if ( ! empty( $tracking_pixel_url )) {
				$template_variables['tracking_pixel_url'] = $tracking_pixel_url;
			}

			$this->add_template_variables(
				$email,
				$template_variables
			);
			

			$this->batch_data[] = $template_variables;
			
			$this->current_batch_size++;
		}

		/**
		 * Set recipient variables
		 * 
		 * @param string $email
		 * @param array $template_variables
		 * 
		 * @since 4.7.0
		 */
		public function add_template_variables( $email, $template_variables = array() ) {
			if ( ! empty( $email ) && ! empty( $template_variables) ) {
				$this->ses_message->addTo( $email );
				$this->ses_message->addTemplateVariables( $email, $template_variables );
			}
		}

		/**
		 * It's the last one, so we can modify the whole body.
		 *
		 * @param array $attachments The array of attachments data.
		 */
		public function set_attachments( $attachments ) {

			if ( empty( $attachments ) ) {
				return;
			}
			if ( ! empty( $attachments ) ) {
				foreach ( $attachments as $attachment_name => $attachment_path ) {
					if ( is_file( $attachment_path ) ) {
						$this->ses_message->addAttachmentFromFile( $attachment_name, $attachment_path );
					}
				}
			}
		}

		/**
		 * Send email using Mailgun API
		 * 
		 * @since 4.7.0
		 */
		public function send_email() {

			try {
				$response = $this->ses_request->SendBulkTemplatedEmail($this->ses_message);
				if ( ! empty( $response['Error'] ) ) {
					ES()->logger->error( 'Error in Email Sending', $this->logger_context );
					ES()->logger->error( $response['Error'], $this->logger_context );
					if ( ! empty( $response['RequestId'] ) ) {
						ES()->logger->error( 'RequestId: ' . $response['RequestId'], $this->logger_context );
					}

					return $this->do_response( 'error', $response['Error'] );
				}
			} catch ( Exception $e ) {
				ES()->logger->error( 'Error in Email Sending', $this->logger_context );

				return $this->do_response( 'error', $e->getMessage() );
			}

			ES()->logger->info( 'Email sent successfully using Amazon SES', $this->logger_context );

			return $this->do_response( 'success' );
		}

		/**
		 * Get variable prefix
		 * 
		 * @return string
		 * 
		 * @since 4.7.2 
		 */
		public function get_variable_prefix() {
			return '{{';
		}

		/**
		 * Get variable suffix
		 * 
		 * @return string
		 * 
		 * @since 4.7.2 
		 */
		public function get_variable_suffix() {
			return '}}';
		}

		/**
		 * Clear batch
		 * 
		 * @since 4.7.0
		 */
		public function clear_batch() {
			
			$this->batch_data                  = array();
			$this->current_batch_size          = 0;
			$this->ses_message->clearTo();
		}

		/**
		 * Delete generated SES data i.e. template
		 *
		 * @since 4.7.2
		 */
		public function clear_email_data() {
			$template_name = $this->ses_message->template_name;
			if ( ! empty( $template_name ) ) {
				$this->delete_email_template( $template_name );
			}

		}

		/**
		 * Can use batch API?
		 *
		 * @param boolean $can_use_batch_api
		 * @param integer $total_recipients
		 * @param array $sender_data
		 * 
		 * @return boolean $can_use_batch_api
		 */
		public function can_use_batch_api( $can_use_batch_api = false, $total_recipients = 0, $sender_data = array() ) {

			// In batch API, since there are mutliple steps paritcular to SES API, create template,send email, delete template to send bulk.
			// Using batch API for one recipient isn't performant, hence we use batch API only if there are more then one recipients.
			if ( $total_recipients > 1 ) {
				// In SES, we can use batch API only if there aren't attachment in the email.
				$can_use_batch_api = empty( $sender_data['attachments'] );
			} else {
				$can_use_batch_api = false;
			}
			
			return $can_use_batch_api;
		}

		/**
		 * Handle throttling 
		 *
		 * @return void
		 * 
		 * @since 5.0.5
		 */
		public function handle_throttling() {
			// We handle SES throttling by making calls to sleep function
			sleep( 5 );
		}
	}

}
