<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'ES_Simple_Email_Service' ) ) {
	/**
	 * Class ES_Simple_Email_Service
	 *
	 * @since 4.6.14
	 */
	class ES_Simple_Email_Service extends IG_ES_Simple_Email_Service {

		public function createTemplate( $template_name, $template_subject, $template_html, $trigger_error = null ) {
			$ses_request = $this->getRequestHandler('POST');
			$action = 'CreateTemplate';
			$ses_request->setParameter('Action', $action);
			$ses_request->setParameter('Template.TemplateName', $template_name );
			$ses_request->setParameter('Template.SubjectPart', $template_subject );
			$ses_request->setParameter('Template.HtmlPart', $template_html );
			
			$ses_response = $ses_request->getResponse();
			if ( false === $ses_response->error && 200 !== $ses_response->code ) {
				$response = array(
					'code' => $ses_response->code,
					'error' => array('Error' => array('message' => 'Unexpected HTTP status')),
				);
				return $response;
			}
			if ( false !== $ses_response->error ) {
				if (( $this->__trigger_errors && ( false !== $trigger_error ) ) || true === $trigger_error) {
					$this->triggerError('createTemplate', $ses_response->error);
					return false;
				}
				return $ses_response;
			}

			$response = array(
				//'MessageId' => (string) $ses_response->body->{"{$action}Result"}->MessageId,
				'RequestId' => (string) $ses_response->body->ResponseMetadata->RequestId,
			);
			return $response;
		}

		public function deleteTemplate( $template_name, $trigger_error = null ) {
			$ses_request = $this->getRequestHandler('POST');
			$action = 'DeleteTemplate';
			$ses_request->setParameter('Action', $action);
			$ses_request->setParameter('TemplateName', $template_name );

			$ses_response = $ses_request->getResponse();
			if ( false === $ses_response->error && 200 !== $ses_response->code ) {
				$response = array(
					'code' => $ses_response->code,
					'error' => array('Error' => array('message' => 'Unexpected HTTP status')),
				);
				return $response;
			}
			if ( false !== $ses_response->error ) {
				if (( $this->__trigger_errors && ( false !== $trigger_error ) ) || true === $trigger_error) {
					$this->triggerError('createTemplate', $ses_response->error);
					return false;
				}
				return $ses_response;
			}

			$response = array(
				//'MessageId' => (string) $ses_response->body->{"{$action}Result"}->MessageId,
				'RequestId' => (string) $ses_response->body->ResponseMetadata->RequestId,
			);
			return $response;
		}

		/**
		* Given a IG_ES_Simple_Email_Service_Message object, submits the message to the service for sending.
		*
		* @param IG_ES_Simple_Email_Service_Message $sesMessage An instance of the message class
		* @param boolean $use_raw_request If this is true or there are attachments to the email `SendRawEmail` call will be used
		* @param boolean $trigger_error Optionally overwrite the class setting for triggering an error (with type check to true/false)
		* @return array An array containing the unique identifier for this message and a separate request id.
		*         Returns false if the provided message is missing any required fields.
		*  @link(AWS SES Response formats, http://docs.aws.amazon.com/ses/latest/DeveloperGuide/query-interface-responses.html)
		*/
		public function SendBulkTemplatedEmail( $sesMessage, $use_raw_request = false, $trigger_error = null) {
			if (!$sesMessage->validate()) {
				$this->triggerError('sendEmail', 'Message failed validation.');
				return false;
			}

			$ses_request = $this->getRequestHandler('POST');
			$action = !empty($sesMessage->attachments) || $use_raw_request ? 'SendRawEmail' : 'SendBulkTemplatedEmail';
			$ses_request->setParameter('Action', $action);

			// Works with both calls
			if (!is_null($sesMessage->configuration_set)) {
				$ses_request->setParameter('ConfigurationSetName', $sesMessage->configuration_set);
			}

			if ( 'SendRawEmail' == $action ) {
				// https://docs.aws.amazon.com/ses/latest/APIReference/API_SendRawEmail.html
				$ses_request->setParameter('RawMessage.Data', $sesMessage->getRawMessage());
			} else {
				$i = 1;
				
				$ses_request->setParameter('DefaultTemplateData', '{}');
				
				foreach ($sesMessage->to as $to) {
					$ses_request->setParameter('Destinations.member.' . $i . '.Destination.ToAddresses.member.1', $sesMessage->encodeRecipients($to));
					if ( ! empty( $sesMessage->template_variables[$to] ) ) {
						$template_data = wp_json_encode( $sesMessage->template_variables[$to] );
						$ses_request->setParameter('Destinations.member.' . $i . '.ReplacementTemplateData', $template_data );
					}
					$i++;
				}

				if (is_array($sesMessage->cc)) {
					$i = 1;
					foreach ($sesMessage->cc as $cc) {
						$ses_request->setParameter('Destination.CcAddresses.member.' . $i, $sesMessage->encodeRecipients($cc));
						$i++;
					}
				}

				if (is_array($sesMessage->bcc)) {
					$i = 1;
					foreach ($sesMessage->bcc as $bcc) {
						$ses_request->setParameter('Destination.BccAddresses.member.' . $i, $sesMessage->encodeRecipients($bcc));
						$i++;
					}
				}

				if (is_array($sesMessage->replyto)) {
					$i = 1;
					foreach ($sesMessage->replyto as $replyto) {
						$ses_request->setParameter('ReplyToAddresses.member.' . $i, $sesMessage->encodeRecipients($replyto));
						$i++;
					}
				}

				$ses_request->setParameter('Source', $sesMessage->encodeRecipients($sesMessage->from));
				$ses_request->setParameter('Template', $sesMessage->template_name);

				if ( null != $sesMessage->returnpath ) {
					$ses_request->setParameter('ReturnPath', $sesMessage->returnpath);
				}

				$i = 1;
				foreach ($sesMessage->message_tags as $key => $value) {
					$ses_request->setParameter('Tags.member.' . $i . '.Name', $key);
					$ses_request->setParameter('Tags.member.' . $i . '.Value', $value);
					$i++;
				}
			}

			$ses_response = $ses_request->getResponse();
			if ( false === $ses_response->error && 200 !== $ses_response->code ) {
				$response = array(
					'code' => $ses_response->code,
					'error' => array('Error' => array('message' => 'Unexpected HTTP status')),
				);
				return $response;
			}
			if ( false !== $ses_response->error ) {
				if (( $this->__trigger_errors && ( false !== $trigger_error ) ) || true === $trigger_error) {
					$this->triggerError('SendBulkTemplatedEmail', $ses_response->error);
					return false;
				}
				return $ses_response;
			}

			
			$response = array(
				'RequestId' => (string) $ses_response->body->ResponseMetadata->RequestId,
			);
			
			$error = (string) $ses_response->body->{"{$action}Result"}->Status->member->Error;

			if ( ! empty( $error ) ) {
				$response['Error'] = $error;
			}

			return $response;
		}
	}
}
