<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'ES_Mailgun_Mailer' ) ) {
	/**
	 * Class ES_Mailgun_Mailer
	 *
	 * @since 4.6.14
	 */
	class ES_Mailgun_Mailer extends ES_Base_Mailer {
		
		/**
		 * Mailer name
		 *
		 * @since 4.8.5
		 * @var
		 */
		public $name = 'Mailgun';

		/**
		 * Mailer Slug
		 *
		 * @since 4.8.5
		 * @var
		 */
		public $slug = 'mailgun';

		/**
		 * Mailgun API Url
		 *
		 * @var string
		 *
		 */
		public $api_url = '';

		/**
		 * Private API Key
		 *
		 * @var string
		 *
		 */
		public $private_api_key = '';

		/**
		 * Domain Name
		 *
		 * @var string
		 *
		 */
		public $domain_name = '';

		/**
		 * Region
		 *
		 * @var string
		 *
		 */
		public $region = 'us';

		/**
		 * Flag to determine whether this mailer support batch sending or not
		 * 
		 * @var boolean
		 * 
		 * @since 4.7.0
		 */
		public $support_batch_sending = true;

		/**
		 * Stores batch sending mode
		 * 
		 * @var boolean
		 * 
		 * @since 4.7.1
		 */
		public $batch_sending_mode = 'multiple';
		
		/**
		 * Batch limit
		 * 
		 * @var boolean
		 * 
		 * @since 4.7.0
		 */
		public $batch_limit = 1000;

		/**
		 * Variable to store batch body data since in case of attachments we need to ovverride $this->body
		 * 
		 * @var boolean
		 * 
		 * @since 4.7.0
		 */
		public $batch_body = array();

		/**
		 * ES_Mailgun_Mailer constructor.
		 */
		public function __construct() {
			parent::__construct();
		}

		/**
		 * Send Email
		 *
		 * @param ES_Message $message
		 *
		 * @return bool|WP_Error
		 */
		public function send( ES_Message $message ) {

			ES()->logger->info( 'Start Sending Email Using Mailgun', $this->logger_context );

			$mailer_data_set = $this->set_mailer_data();
			if ( is_wp_error( $mailer_data_set ) ) {
				return $mailer_data_set;
			}
			
			$this->set_recipients(
				array(
					'to'  => array( $message->to ),
				)
			);
			$this->set_from( $message->from, $message->from_name );
			$this->set_subject( $message->subject );
			$this->set_content(
				array(
					'html' => $message->body,
				)
			);

			/*
			* In some cases we will need to modify the internal structure
			* of the body content, if attachments are present.
			* So lets make this call the last one.
			*/
			if ( $message->attachments ) {
				$this->set_attachments( $message->attachments );
			}

			$response = $this->send_email();
			return $response;
		}

		/**
		 * Set mailer data
		 * 
		 * @since 4.7.0
		 */
		public function set_mailer_data() {

			// Reset body and headers.
			$this->reset_mailer_data();
			
			$ig_es_mailer_settings = get_option( 'ig_es_mailer_settings', array() );
			if ( ES()->is_const_defined( 'mailgun', 'private_api_key' ) ) {
				$this->private_api_key = ES()->get_const_value( 'mailgun', 'private_api_key' );
			} else {
				$this->private_api_key = ! empty( $ig_es_mailer_settings['mailgun']['private_api_key'] ) ? $ig_es_mailer_settings['mailgun']['private_api_key'] : '';
			}

			if ( empty( $this->private_api_key ) ) {
				return $this->do_response( 'error', __( 'Private API key is empty.', 'email-subscribers' ) );
			}

			if ( ES()->is_const_defined( 'mailgun', 'domain_name' ) ) {
				$this->domain_name = ES()->get_const_value( 'mailgun', 'domain_name' );
			} else {
				$this->domain_name = ! empty( $ig_es_mailer_settings['mailgun']['domain_name'] ) ? $ig_es_mailer_settings['mailgun']['domain_name'] : '';
			}

			if ( empty( $this->domain_name ) ) {
				return $this->do_response( 'error', __( 'Domain name is empty', 'email-subscribers' ) );
			} 

			if ( ES()->is_const_defined( 'mailgun', 'region' ) ) {
				$this->region = ES()->get_const_value( 'mailgun', 'region' );
			} else {
				$this->region = ! empty( $ig_es_mailer_settings['mailgun']['region'] ) ? $ig_es_mailer_settings['mailgun']['region'] : '';
			}

			if ( empty( $this->region ) ) {
				return $this->do_response( 'error', __( 'Region is empty', 'email-subscribers' ) );
			}

			if ( 'eu' === $this->region ) {
				$this->api_url = 'https://api.eu.mailgun.net/v3/';
			} else {
				$this->api_url = 'https://api.mailgun.net/v3/';
			}

			/*
			* Append the url with a domain name
			*/
			$this->api_url .= sanitize_text_field( $this->domain_name . '/messages' );

			$this->set_header( 'Authorization', 'Basic ' . base64_encode( 'api:' . $this->private_api_key ) );

			$this->set_tracking_options();
		}

		/**
		 * Set from
		 */
		public function set_from( $email, $name = '' ) {

			if ( ! filter_var( $email, FILTER_VALIDATE_EMAIL ) ) {
				return;
			}

			if ( ! empty( $name ) ) {
				$this->set_body_param(
					array(
						'from' => $name . ' <' . $email . '>',
					)
				);
			} else {
				$this->set_body_param(
					array(
						'from' => $email,
					)
				);
			}
		}

		/**
		 * Set reply to
		 * 
		 * @param string $email
		 * 
		 * @since 4.6.7
		 */
		public function set_reply_to( $email ) {

			if ( ! filter_var( $email, FILTER_VALIDATE_EMAIL ) ) {
				return;
			}
			
			$this->set_body_param(
				array(
					'h:Reply-To' => $email,
				)
			);
		}

		/**
		 * Set list unsubscribe header
		 * 
		 * @since 4.7.2
		 */
		public function set_list_unsubscribe_header() {

			$this->set_body_param(
				array(
					'h:List-Unsubscribe'      => '%recipient.list_unsubscribe_header%',
					'h:List-Unsubscribe-Post' => 'List-Unsubscribe=One-Click',
				)
			);
		}

		/**
		 * Set recipients
		 */
		public function set_recipients( $recipients ) {

			if ( empty( $recipients ) ) {
				return;
			}

			$default = array( 'to' );

			foreach ( $recipients as $kind => $emails ) {
				if (
					! in_array( $kind, $default, true ) ||
					empty( $emails ) ||
					! is_array( $emails )
				) {
					continue;
				}

				$data = array();

				foreach ( $emails as $email ) {

					if ( ! filter_var( $email, FILTER_VALIDATE_EMAIL ) ) {
						continue;
					}

					$data[] = $email;
				}

				if ( ! empty( $data ) ) {
					$this->set_body_param(
						array(
							$kind => implode( ', ', $data ),
						)
					);
				}
			}
		}

		/**
		 * Set email content
		 */
		public function set_content( $content ) {

			if ( is_array( $content ) ) {

				$default = array( 'html' );

				foreach ( $content as $type => $content_body ) {
					if (
						! in_array( $type, $default, true ) ||
						empty( $content_body )
					) {
						continue;
					}

					$this->set_body_param(
						array(
							$type => $content_body,
						)
					);
				}
			} else {

				$type = 'html';

				if ( ! empty( $content ) ) {
					$this->set_body_param(
						array(
							$type => $content,
						)
					);
				}
			}
		}

		/**
		 * Set email data
		 * e.g. Sender email, name
		 * 
		 * @since 4.7.2 
		 */
		public function set_email_data( $email_data = array() ) {

			$sender_email   = ! empty( $email_data['sender_email'] ) ? $email_data['sender_email']    : '';
			$sender_name    = ! empty( $email_data['sender_name'] ) ? $email_data['sender_name']      : '';
			$reply_to_email = ! empty( $email_data['reply_to_email'] ) ? $email_data['reply_to_email']: '';
			$subject        = ! empty( $email_data['subject'] ) ? $email_data['subject']              : '';
			$content        = ! empty( $email_data['content'] ) ? $email_data['content']              : '';

			$this->set_from( $sender_email, $sender_name );
			$this->set_reply_to( $reply_to_email );
			$this->set_subject( $subject );
			$this->set_content( array(
				'html' => $content
			) );
		}

		/**
		 * It's the last one, so we can modify the whole body.
		 *
		 * @param array $attachments The array of attachments data.
		 */
		public function set_attachments( $attachments ) {

			if ( empty( $attachments ) ) {
				return;
			}

			$payload = '';
			$data    = array();

			foreach ( $attachments as $attachment_name => $attachment_path ) {
				$file = false;
				try {
					if ( is_file( $attachment_path ) && is_readable( $attachment_path ) ) {
						$file = file_get_contents( $attachment_path );
					}
				} catch ( Exception $e ) {
					$file = false;
				}

				if ( false === $file ) {
					continue;
				}

				$data[] = array(
					'content' => $file,
					'name'    => $attachment_name,
				);
			}

			if ( ! empty( $data ) ) {

				// First, generate a boundary for the multipart message.
				$boundary = hash( 'crc32', md5( uniqid( 'boundary', true ) ) );

				if ( ! empty( $this->body['recipient-variables'] ) && is_array( $this->body['recipient-variables'] ) ) {
					$this->body['recipient-variables'] = wp_json_encode( $this->body['recipient-variables'] );
				}

				// Iterate through pre-built params and build a payload.
				foreach ( $this->body as $key => $value ) {
					if ( is_array( $value ) ) {
						foreach ( $value as $child_key => $child_value ) {
							$payload .= '--' . $boundary;
							$payload .= "\r\n";
							$payload .= 'Content-Disposition: form-data; name="' . $key . "\"\r\n\r\n";
							$payload .= $child_value;
							$payload .= "\r\n";
						}
					} else {
						$payload .= '--' . $boundary;
						$payload .= "\r\n";
						$payload .= 'Content-Disposition: form-data; name="' . $key . '"' . "\r\n\r\n";
						$payload .= $value;
						$payload .= "\r\n";
					}
				}

				// Now iterate through our attachments, and add them too.
				foreach ( $data as $key => $attachment ) {
					$payload .= '--' . $boundary;
					$payload .= "\r\n";
					$payload .= 'Content-Disposition: form-data; name="attachment[' . $key . ']"; filename="' . $attachment['name'] . '"' . "\r\n\r\n";
					$payload .= $attachment['content'];
					$payload .= "\r\n";
				}

				$payload .= '--' . $boundary . '--';

				$this->batch_body = $this->body;

				// Redefine the body the "dirty way".
				$this->body = $payload;

				$this->set_header( 'Content-Type', 'multipart/form-data; boundary=' . $boundary );
			}
		}

		/**
		 * Add into batch
		 * 
		 * @param string $email
		 * @param array $merge_tags
		 * 
		 * @since 4.7.0
		 */
		public function add_into_batch( $email, $merge_tags = array(), $message = null ) {
			$this->set_recipients(
				array(
					'to'  => array( $email ),
				)
			);

			$name        = ig_es_get_data( $merge_tags, 'name', '' );
			$first_name  = ig_es_get_data( $merge_tags, 'first_name', '' );
			$last_name   = ig_es_get_data( $merge_tags, 'last_name', '' );
			$list_name   = ig_es_get_data( $merge_tags, 'list_name', '' );
			$hash        = ig_es_get_data( $merge_tags, 'hash', '' );
			$contact_id  = ig_es_get_data( $merge_tags, 'contact_id', 0 );
			$campaign_id = ig_es_get_data( $merge_tags, 'campaign_id', 0 );
			$message_id  = ig_es_get_data( $merge_tags, 'message_id', 0 );
			$list_ids    = ig_es_get_data( $merge_tags, 'list_ids', '' );

			$link_data = array(
				'message_id'  => $message_id,
				'campaign_id' => $campaign_id,
				'contact_id'  => $contact_id,
				'email'       => $email,
				'guid'        => $hash,
				'list_ids'	  => $list_ids,
			);
			
			$subscribe_link     = ES()->mailer->get_subscribe_link( $link_data );
			$unsubscribe_link   = ES()->mailer->get_unsubscribe_link( $link_data );
			$link_variables     = ES()->mailer->get_link_variable( $contact_id );
			$tracking_pixel_url = ES()->mailer->get_tracking_pixel_url( $link_data );

			$recipient_variables = array(
				'name'             => $name,
				'first_name'       => $first_name,
				'last_name'        => $last_name,
				'list_name'        => $list_name,
				'hash'             => $hash,
				'email'            => $email,
				'contact_id'       => $contact_id,
				'campaign_id'      => $campaign_id,
				'message_id'       => $message_id,
				'list_ids'         => $list_ids,
				'subscribe_link'   => $subscribe_link,
				'unsubscribe_link' => $unsubscribe_link,
			);

			if ( ! empty( $link_variables ) ) {
				$recipient_variables = array_merge( $recipient_variables, $link_variables );
			}

			if ( ! empty( $tracking_pixel_url )) {
				$recipient_variables['tracking_pixel_url'] = $tracking_pixel_url;
			}

			// Check if it is an campaign email and headers are enabled on the site.
			$list_unsubscribe_header = ES()->mailer->get_list_unsubscribe_header( $email );
			if ( $list_unsubscribe_header ) {
				$recipient_variables['list_unsubscribe_header'] = $list_unsubscribe_header;
			}

			$this->set_recipient_variable(
				$email,
				$recipient_variables
			);
			

			$this->batch_data[] = $recipient_variables;
			
			$this->current_batch_size++;
		}

		/**
		 * Convert ES tags to mailer tags
		 * 
		 * @param string $string
		 * 
		 * @return string $string
		 * 
		 * @since 4.7.0
		 */
		public function convert_es_tags_to_mailer_tags( $string = '' ) {
			return ES_Common::replace_keywords_with_fallback( $string, array(
				'NAME'             => '%recipient.name%',
				'FIRSTNAME'        => '%recipient.first_name%',
				'LASTNAME'         => '%recipient.last_name%',
				'EMAIL'            => '%recipient.email%',
				'UNSUBSCRIBE-LINK' => '%recipient.unsubscribe_link%',
				'SUBSCRIBE-LINK'   => '%recipient.subscribe_link%',
			) );
		}

		/**
		 * Set recipient variables
		 * 
		 * @param string $email
		 * @param array $recipient_variables
		 * 
		 * @since 4.7.0
		 */
		public function set_recipient_variable( $email, $recipient_variables = array() ) {
			if ( ! empty( $email ) && ! empty( $recipient_variables) ) {
				$this->set_body_param(
					array(
						'recipient-variables' => array(
							$email => $recipient_variables
						)
					)
				);
			}
		}

		/**
		 * Send email using Mailgun API
		 * 
		 * @since 4.7.0
		 */
		public function send_email() {
			$params = array_merge_recursive( $this->get_default_params(), array(
				'headers' => $this->get_headers(),
				'body'    => $this->get_body(),
			));
			
			$response = wp_remote_post( $this->api_url, $params );
			if ( ! is_wp_error( $response ) ) {
				$body = ! empty( $response['body'] ) && ES_Common::is_valid_json( $response['body'] ) ? json_decode( $response['body'], true ) : '';
				if ( ! empty( $body ) ) {
					// If we have received an id in response then email is successfully queued at mailgun server.
					if ( ! empty( $body['id'] ) ) {
						return $this->do_response( 'success' );
					} else {
						if ( ! empty( $body['message'] ) ) {
							$error_message = $body['message'];
						} else {
							$error_message = __( 'An unknown error has occured. Please try again later.', 'email-subscribers' );
						}
						return $this->do_response( 'error', $error_message );
					}
				} else {
					return $this->do_response( 'error', wp_remote_retrieve_response_message( $response ) );
				}
			} else {
				$error_message = $response->get_error_message();
				return $this->do_response( 'error', $error_message );
			}

			ES()->logger->info( 'Email Sent Successfully Using Mailgun', $this->logger_context );

			return $this->do_response( 'success' );
		}

		/**
		 * Clear batch
		 * 
		 * @since 4.7.0
		 */
		public function clear_batch() {
			if ( ! empty( $this->batch_body ) ) {
				$this->body = $this->batch_body;
			}
			$this->body['to']                  = array();
			$this->body['recipient-variables'] = array();
			$this->batch_data                  = array();
			$this->current_batch_size          = 0;
		}

		/**
		 * Get the email body.
		 *
		 * @return string|array
		 *
		 * @since 4.7.0
		 */
		public function get_body() {

			if ( ! empty( $this->body['recipient-variables'] ) ) {
				$this->body['recipient-variables'] = wp_json_encode($this->body['recipient-variables'] );
			}

			return apply_filters( 'ig_es_mailer_get_body', $this->body, $this );
		}

		/**
		 * Get placeholder variable name string for Mailgun
		 * 
		 * @return string $variable_string
		 * 
		 * @since 4.7.0
		 */
		public function get_variable_string( $variable_name = '' ) {
			$variable_string = 'recipient';
			if ( ! empty( $variable_name ) ) {
				$variable_string .= '.' . $variable_name;
			}
			return $variable_string;
		}

		/**
		 * Set open/click tracking options
		 * 
		 * @since 4.7.0
		 */
		public function set_tracking_options() {

			$this->set_body_param(
				array(
					'o:tracking-opens'  => ES()->mailer->can_track_open() ? 'yes' : 'no',
					'o:tracking-clicks' => ES()->mailer->can_track_clicks() ? 'yes' : 'no',
				)
			);
		}

		/**
		 * Get variable prefix
		 * 
		 * @return string
		 * 
		 * @since 4.7.2 
		 */
		public function get_variable_prefix() {
			return '%';
		}

		/**
		 * Get variable suffix
		 * 
		 * @return string
		 * 
		 * @since 4.7.2 
		 */
		public function get_variable_suffix() {
			return '%';
		}
	}
}
