<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'ES_Postmark_Mailer' ) ) {
	/**
	 * Class ES_Postmark_Mailer
	 *
	 * @since 4.6.14
	 */
	class ES_Postmark_Mailer extends ES_Base_Mailer {

		/**
		 * Mailer name
		 *
		 * @since 4.8.5
		 * @var
		 */
		public $name = 'Postmark';

		/**
		 * Mailer Slug
		 *
		 * @since 4.8.5
		 * @var
		 */
		public $slug = 'postmark';

		/**
		 * Postmark API Url
		 *
		 * @var string
		 *
		 */
		public $api_url = '';

		/**
		 * Private API Key
		 *
		 * @var string
		 *
		 */
		public $private_api_key = '';

		/**
		 * Domain Name
		 *
		 * @var string
		 *
		 */
		public $domain_name = '';

		/**
		 * Region
		 *
		 * @var string
		 *
		 */
		public $region = 'us';

		/**
		 * Flag to determine whether this mailer support batch sending or not
		 * 
		 * @var boolean
		 * 
		 * @since 4.7.1
		 */
		public $support_batch_sending = true;

		/**
		 * Stores batch sending mode
		 * 
		 * @var boolean
		 * 
		 * @since 4.7.1
		 */
		public $batch_sending_mode = 'single';
		
		/**
		 * Batch limit
		 * 
		 * @var boolean
		 * 
		 * @since 4.7.1
		 */
		public $batch_limit = 500;

		/**
		 * Variable to store batch body data since in case of attachments we need to ovverride $this->body
		 * 
		 * @var boolean
		 * 
		 * @since 4.7.1
		 */
		public $batch_body = array();

		/**
		 * ES_Postmark_Mailer constructor.
		 */
		public function __construct() {
			parent::__construct();
		}

		/**
		 * Set mailer data
		 * 
		 * @since 4.7.1
		 */
		public function set_mailer_data( $use_batch_api = true ) {

			// Reset body and headers.
			$this->reset_mailer_data();
			
			$ig_es_mailer_settings = get_option( 'ig_es_mailer_settings', array() );
			if ( ES()->is_const_defined( 'postmark', 'api_token' ) ) {
				$this->api_token = ES()->get_const_value( 'postmark', 'api_token' );
			} else {
				$this->api_token = ! empty( $ig_es_mailer_settings['postmark']['api_token'] ) ? $ig_es_mailer_settings['postmark']['api_token'] : '';
			}

			if ( empty( $this->api_token ) ) {
				return $this->do_response( 'error', __( 'API token is empty.', 'email-subscribers' ) );
			}

			if ( $use_batch_api ) {
				$this->api_url = 'https://api.postmarkapp.com/email/batch';
			} else {
				$this->api_url = 'https://api.postmarkapp.com/email/';
			}

			$this->set_header( 'Accept', 'application/json' );
			$this->set_header( 'Content-Type', 'application/json' );
			$this->set_header( 'X-Postmark-Server-Token', $this->api_token );
			
		}

		/**
		 * It's the last one, so we can modify the whole body.
		 *
		 * @param array $attachments The array of attachments data.
		 */
		public function get_attachments( $attachments ) {
			$data = array();

			if ( empty( $attachments ) ) {
				return $data;
			}

			foreach ( $attachments as $attachment_name => $attachment_path ) {
				$file = false;

				try {
					if ( is_file( $attachment_path ) && is_readable( $attachment_path ) ) {
						$file = file_get_contents( $attachment_path );
					}
				} catch ( Exception $e ) {
					$file = false;
				}

				if ( false === $file ) {
					continue;
				}

				$filetype = mime_content_type( $attachment_path );

				$data[] = array(
					'Name'    => empty( $attachment_name ) ? 'file-' . wp_hash( microtime() ) . '.' . $filetype : trim( $attachment_name ), // required string, no CRLF.
					'Content'     => base64_encode( $file ), // string, 1 character.
					'ContentType'        => $filetype, // string, no ;, no CRLF.
				);
			}

			return $data;
		}

		/**
		 * Add into batch
		 * 
		 * @param string $email
		 * @param array $merge_tags
		 * 
		 * @since 4.7.1
		 */
		public function add_into_batch( $email, $merge_tags = array(), $message = null ) {
			
			$message_data = array(
				'From'          => $message->from,
				'To'            => $email,
				'ReplyTo'       => $message->reply_to_email,
				'Subject'       => $message->subject,
				'HtmlBody'      => $message->body,
				'TrackOpens'    => ES()->mailer->can_track_open(),
				'TrackLinks'    => ES()->mailer->can_track_clicks() ? 'HtmlOnly' : 'None',
				'MessageStream' => 'outbound',
			);

			$list_unsubscribe_header = ES()->mailer->get_list_unsubscribe_header( $email );
			if ( ! empty( $list_unsubscribe_header ) ) {
				$message_data['headers'] = array(
					array(
						'Name' => 'List-Unsubscribe',
						'Value' => $list_unsubscribe_header
					),
					array(
						'Name' => 'List-Unsubscribe-Post',
						'Value' => 'List-Unsubscribe=One-Click'
					),
				);
			}

			if ( ! empty( $message->attachments ) ) {
				$attachments = $this->get_attachments( $message->attachments );
				if ( ! empty( $attachments ) ) {
					$message_data['Attachments'] = $attachments;
				}
			}

			$this->set_body_param(
				array(
					$message_data
				)
			);

			$contact_id = ig_es_get_data( $merge_tags, 'contact_id', 0 );

			$this->batch_data[] = array(
				'contact_id' => $contact_id,
			);
			
			$this->current_batch_size++;
		}

		/**
		 * Send single recipient emails
		 * 
		 * @since 4.7.2
		 */
		public function send( ES_Message $message ) {
			$response = $this->set_mailer_data( false );
			
			// Error setting up mailer?
			if ( is_wp_error( $response ) ) {
				return $response;
			}

			$message_data = array(
				'From'          => $message->from,
				'To'            => $message->to,
				'ReplyTo'       => $message->reply_to_email,
				'Subject'       => $message->subject,
				'HtmlBody'      => $message->body,
				'TrackOpens'    => ES()->mailer->can_track_open(),
				'TrackLinks'    => ES()->mailer->can_track_clicks() ? 'HtmlOnly' : 'None',
				'MessageStream' => 'outbound',
			);

			$list_unsubscribe_header = ES()->mailer->get_list_unsubscribe_header( $message->to );
			if ( ! empty( $list_unsubscribe_header ) ) {
				$message_data['headers'] = array(
					array(
						'Name' => 'List-Unsubscribe',
						'Value' => $list_unsubscribe_header
					),
					array(
						'Name' => 'List-Unsubscribe-Post',
						'Value' => 'List-Unsubscribe=One-Click'
					),
				);
			}

			if ( ! empty( $message->attachments ) ) {
				$attachments = $this->get_attachments( $message->attachments );
				if ( ! empty( $attachments ) ) {
					$message_data['Attachments'] = $attachments;
				}
			}

			$this->set_body_param( $message_data );

			$response = $this->send_email();
			return $response;
		}

		/**
		 * Send email using Postmark API
		 * 
		 * @since 4.7.1
		 */
		public function send_email() {
			$params = array_merge_recursive( $this->get_default_params(), array(
				'headers' => $this->get_headers(),
				'body'    => $this->get_body(),
			));

			$response = wp_remote_post( $this->api_url, $params );
			
			if ( ! is_wp_error( $response ) ) {
				$body = ! empty( $response['body'] ) && ES_Common::is_valid_json( $response['body'] ) ? json_decode( $response['body'], true ) : '';
				if ( ! empty( $body ) ) {

					// Since all the email will be having same kind of error in a batch, we are checking email sending status for first recipient email only.
					if ( isset( $body[0] ) ) {
						$response_data = $body[0];
					} else {
						$response_data = $body;
					}

					// If ErrorCode is empty then email is successfully sent at postmark server.
					if ( empty( $response_data['ErrorCode'] ) ) {
						return $this->do_response( 'success' );
					} else {
						if ( ! empty( $response_data['Message'] ) ) {
							$error_message = $response_data['Message'];
						} else {
							$error_message = __( 'An unknown error has occured. Please try again later.', 'email-subscribers' );
						}
						return $this->do_response( 'error', $error_message );
					}
				} else {
					return $this->do_response( 'error', wp_remote_retrieve_response_message( $response ) );
				}
			} else {
				$error_message = $response->get_error_message();
				return $this->do_response( 'error', $error_message );
			}

			ES()->logger->info( 'Email Sent Successfully Using Postmark', $this->logger_context );

			return $this->do_response( 'success' );
		}

		/**
		 * Clear batch
		 * 
		 * @since 4.7.1
		 */
		public function clear_batch() {
			$this->body               = array();
			$this->batch_data         = array();
			$this->current_batch_size = 0;
		}

		/**
		 * Get the email body.
		 *
		 * @return string|array
		 *
		 * @since 4.7.1
		 */
		public function get_body() {

			return wp_json_encode( $this->body );
		}
	}
}
