<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'ES_Sendinblue_Mailer' ) ) {
	/**
	 * Class ES_Sendinblue_Mailer
	 *
	 * @since 5.3.2
	 */
	class ES_Sendinblue_Mailer extends ES_Base_Mailer {

		/**
		 * Mailer name
		 *
		 * @var
		 */
		public $name = 'Sendinblue';

		/**
		 * Mailer Slug
		 *
		 * @var
		 */
		public $slug = 'sendinblue';

		/**
		 * Sendinblue API Url
		 *
		 * @var string
		 *
		 */
		public $api_url = 'https://api.sendinblue.com/v3/smtp/email';

		/**
		 * Private API Key
		 *
		 * @var string
		 *
		 */
		public $private_api_key = '';

		/**
		 * Domain Name
		 *
		 * @var string
		 *
		 */
		public $domain_name = '';

		/**
		 * Region
		 *
		 * @var string
		 *
		 */
		public $region = 'us';

		/**
		 * Flag to determine whether this mailer support batch sending or not
		 *
		 * @var boolean
		 */
		public $support_batch_sending = true;

		/**
		 * Stores batch sending mode
		 *
		 * @var boolean
		 */
		public $batch_sending_mode = 'multiple';

		/**
		 * Batch limit
		 *
		 * @var boolean
		 */
		public $batch_limit = 1000;

		/**
		 * Variable to store batch body data since in case of attachments we need to ovverride $this->body
		 *
		 * @var boolean
		 */
		public $batch_body = array();

		/**
		 * ES_Sendinblue_Mailer constructor.
		 */
		public function __construct() {
			parent::__construct();
		}

		/**
		 * Set mailer data
		 */
		public function set_mailer_data( $use_batch_api = true ) {

			// Reset body and headers.
			$this->reset_mailer_data();

			$ig_es_mailer_settings = get_option( 'ig_es_mailer_settings', array() );
			if ( ES()->is_const_defined( 'sendinblue', 'api_token' ) ) {
				$this->api_token = ES()->get_const_value( 'sendinblue', 'api_token' );
			} else {
				$this->api_token = ! empty( $ig_es_mailer_settings['sendinblue']['api_token'] ) ? $ig_es_mailer_settings['sendinblue']['api_token'] : '';
			}

			if ( empty( $this->api_token ) ) {
				return $this->do_response( 'error', __( 'API token is empty.', 'email-subscribers' ) );
			}

			$this->set_header( 'Accept', 'application/json' );
			$this->set_header( 'Content-Type', 'application/json' );
			$this->set_header( 'api-key', $this->api_token );
		}

		/**
		 * It's the last one, so we can modify the whole body.
		 *
		 * @param array $attachments The array of attachments data.
		 */
		public function get_attachments( $attachments ) {
			$data = array();

			if ( empty( $attachments ) ) {
				return $data;
			}

			foreach ( $attachments as $attachment_name => $attachment_path ) {
				$file = false;

				try {
					if ( is_file( $attachment_path ) && is_readable( $attachment_path ) ) {
						$file = file_get_contents( $attachment_path );
					}
				} catch ( Exception $e ) {
					$file = false;
				}

				if ( false === $file ) {
					continue;
				}

				$filetype = mime_content_type( $attachment_path );

				$data[] = array(
					'name'    => empty( $attachment_name ) ? 'file-' . wp_hash( microtime() ) . '.' . $filetype : trim( $attachment_name ),
					// required string, no CRLF.
					'content' => base64_encode( $file ),
					// string, 1 character.
				);
			}

			return $data;
		}

		/**
		 * Add into batch
		 *
		 * @param string $email
		 * @param array $merge_tags
		 */
		public function add_into_batch( $email, $merge_tags = array(), $message = null ) {
			$name        = ig_es_get_data( $merge_tags, 'name', '' );
			$first_name  = ig_es_get_data( $merge_tags, 'first_name', '' );
			$last_name   = ig_es_get_data( $merge_tags, 'last_name', '' );
			$list_name   = ig_es_get_data( $merge_tags, 'list_name', '' );
			$hash        = ig_es_get_data( $merge_tags, 'hash', '' );
			$contact_id  = ig_es_get_data( $merge_tags, 'contact_id', 0 );
			$campaign_id = ig_es_get_data( $merge_tags, 'campaign_id', 0 );
			$message_id  = ig_es_get_data( $merge_tags, 'message_id', 0 );
			$list_ids    = ig_es_get_data( $merge_tags, 'list_ids', '' );

			$link_data = array(
				'message_id'  => $message_id,
				'campaign_id' => $campaign_id,
				'contact_id'  => $contact_id,
				'email'       => $email,
				'guid'        => $hash,
				'list_ids'    => $list_ids,
			);

			$subscribe_link     = ES()->mailer->get_subscribe_link( $link_data );
			$unsubscribe_link   = ES()->mailer->get_unsubscribe_link( $link_data );
			$link_variables     = ES()->mailer->get_link_variable( $contact_id );
			$tracking_pixel_url = ES()->mailer->get_tracking_pixel_url( $link_data );

			$contact_data = array(
				'name'             => $name,
				'first_name'       => $first_name,
				'last_name'        => $last_name,
				'list_name'        => $list_name,
				'hash'             => $hash,
				'email'            => $email,
				'contact_id'       => $contact_id,
				'campaign_id'      => $campaign_id,
				'message_id'       => $message_id,
				'list_ids'         => $list_ids,
				'subscribe_link'   => $subscribe_link,
				'unsubscribe_link' => $unsubscribe_link,
			);

			if ( ! empty( $link_variables ) ) {
				$contact_data = array_merge( $contact_data, $link_variables );
			}

			if ( ! empty( $tracking_pixel_url ) ) {
				$contact_data['tracking_pixel_url'] = $tracking_pixel_url;
			}

			// Check if it is an campaign email and headers are enabled on the site.
			$list_unsubscribe_header = ES()->mailer->get_list_unsubscribe_header( $email );
			if ( $list_unsubscribe_header ) {
				$contact_data['list_unsubscribe_header'] = $list_unsubscribe_header;
			}

			$recipient_variables = array();

			foreach ( $contact_data as $data_key => $data_value ) {
				$recipient_variables[ $data_key ] = $data_value;
			}

			$this->set_recipients(
				array(
					'to'     => array( 'email' => $email, 'name' => $name ),
					'params' => $recipient_variables,
				)
			);

			$this->batch_data[] = $contact_data;

			$this->current_batch_size ++;
		}

		/**
		 * Set email data
		 * e.g. Sender email, name
		 */
		public function set_email_data( $email_data = array() ) {
			$sender_email   = ! empty( $email_data['sender_email'] ) ? $email_data['sender_email'] : '';
			$sender_name    = ! empty( $email_data['sender_name'] ) ? $email_data['sender_name'] : '';
			$reply_to_email = ! empty( $email_data['reply_to_email'] ) ? $email_data['reply_to_email'] : '';
			$subject        = ! empty( $email_data['subject'] ) ? $email_data['subject'] : '';
			$content        = ! empty( $email_data['content'] ) ? $email_data['content'] : '';

			$this->set_from( $sender_email, $sender_name );
			$this->set_reply_to( $reply_to_email );
			$this->set_subject( $subject );
			$this->set_content( $content );
		}

		/**
		 * Set from
		 */
		public function set_from( $email, $name = '' ) {

			if ( ! filter_var( $email, FILTER_VALIDATE_EMAIL ) ) {
				return;
			}
			$sender = array( 'email' => $email );
			if ( ! empty( $name ) ) {
				$sender['name'] = $name;
			}
			$this->set_body_param(
				array(
					'sender' => $sender,
				)
			);
		}

		/**
		 * Set reply to
		 *
		 * @param string $email
		 */
		public function set_reply_to( $email ) {

			if ( ! filter_var( $email, FILTER_VALIDATE_EMAIL ) ) {
				return;
			}

			$this->set_body_param(
				array(
					'replyTo' => array(
						'email' => $email
					),
				)
			);
		}

		/**
		 * Get variable prefix
		 *
		 * @return string
		 */
		public function get_variable_prefix() {
			return '{{';
		}

		/**
		 * Get variable suffix
		 *
		 * @return string
		 */
		public function get_variable_suffix() {
			return '}}';
		}

		/**
		 * Convert ES tags to mailer tags
		 *
		 * @param string $string
		 * @return string $string
		 */
		public function convert_es_tags_to_mailer_tags( $string = '' ) {
			return ES_Common::replace_keywords_with_fallback( $string, array(
				'NAME'             => '{{params.name | default : "%%fallback%%"}}',
				'FIRSTNAME'        => '{{params.first_name | default : "%%fallback%%"}}',
				'LASTNAME'         => '{{params.last_name}}',
				'EMAIL'            => '{{params.email}}',
				'UNSUBSCRIBE-LINK' => '{{params.unsubscribe_link}}',
				'SUBSCRIBE-LINK'   => '{{params.subscribe_link}}',
			) );
		}

		/**
		 * Set recipients
		 */
		public function set_recipients( $recipients ) {

			if ( empty( $recipients ) ) {
				return;
			}
			$data = array();
			foreach ( $recipients as $type => $recipient_data ) {
				$data[ $type ] = array();
				if ( 'to' == $type ) {
					if ( is_array( $recipient_data ) && isset( $recipient_data['email'] ) && filter_var( $recipient_data['email'], FILTER_VALIDATE_EMAIL ) ) {
						array_push( $data[ $type ], $recipient_data );
					}
				} else {
					$data[ $type ] = $recipient_data;
				}
			}
			if ( ! empty( $data ) ) {
				$this->set_body_param( array( 'messageVersions' => array( $data ), ) );
			}
		}

		/**
		 * Set email content
		 */
		public function set_content( $content ) {
			if ( empty( $content ) ) {
				return;
			}
			$this->set_body_param(
				array(
					'htmlContent' => $content,
				)
			);
		}

		/**
		 * Send single recipient emails
		 */
		public function send( ES_Message $message ) {
			$response = $this->set_mailer_data( false );

			// Error setting up mailer?
			if ( is_wp_error( $response ) ) {
				return $response;
			}

			$to = array( 'email' => $message->to );
			if ( ! empty( $message->to_name ) ) {
				$to['name'] = $message->to_name;
			}


			$message_data = array();
			$this->set_subject( $message->subject );
			$this->set_content($message->body);
			$this->set_from( $message->from, $message->from_name );
			$this->set_reply_to( $message->reply_to_email );
			$this->set_recipients( array(
				'to' => $to,
			) );

			$list_unsubscribe_header = ES()->mailer->get_list_unsubscribe_header( $message->to );
			if ( ! empty( $list_unsubscribe_header ) ) {
				$message_data['headers'] = array(
					'List-Unsubscribe'      => $list_unsubscribe_header,
					'List-Unsubscribe-Post' => 'List-Unsubscribe=One-Click'
				);
			}

			if ( ! empty( $message->attachments ) ) {
				$attachments = $this->get_attachments( $message->attachments );
				if ( ! empty( $attachments ) ) {
					$message_data['attachment'] = $attachments;
				}
			}

			$this->set_body_param( $message_data );

			return $this->send_email();
		}

		/**
		 * Send email using sendinblue API
		 */
		public function send_email() {
			$params = array_merge_recursive( $this->get_default_params(), array(
				'headers' => $this->get_headers(),
				'body'    => $this->get_body(),
			) );

			$response = wp_remote_post( $this->api_url, $params );

			if ( ! is_wp_error( $response ) ) {
				$body = ! empty( $response['body'] ) && ES_Common::is_valid_json( $response['body'] ) ? json_decode( $response['body'], true ) : '';
				if ( ! empty( $body ) ) {

					// Since all the email will be having same kind of error in a batch, we are checking email sending status for first recipient email only.
					if ( isset( $body[0] ) ) {
						$response_data = $body[0];
					} else {
						$response_data = $body;
					}

					// If ErrorCode is empty then email is successfully sent at sendinblue server.
					if ( empty( $response_data['ErrorCode'] ) ) {
						ES()->logger->info( 'Email Sent Successfully Using Sendinblue', $this->logger_context );

						return $this->do_response( 'success' );
					} else {
						if ( ! empty( $response_data['Message'] ) ) {
							$error_message = $response_data['Message'];
						} else {
							$error_message = __( 'An unknown error has occured. Please try again later.', 'email-subscribers' );
						}

						return $this->do_response( 'error', $error_message );
					}
				} else {
					return $this->do_response( 'error', wp_remote_retrieve_response_message( $response ) );
				}
			} else {
				$error_message = $response->get_error_message();

				return $this->do_response( 'error', $error_message );
			}
		}

		/**
		 * Clear batch
		 */
		public function clear_batch() {
			$this->body               = array();
			$this->batch_data         = array();
			$this->current_batch_size = 0;
		}

		/**
		 * Get the email body.
		 *
		 * @return false|string
		 */
		public function get_body() {
			return wp_json_encode( $this->body );
		}
	}
}
