<?php
/**
 * Adds extra fields to existing add to list action
 *
 * @since       4.5.3
 * @version     1.0
 * @package     Email Subscribers
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/***
 * ES_Pro_Action_Add_To_List class.
 *
 * @since 4.5.3
 */
class ES_Pro_Action_Add_To_List {
	
	/**
	 * Method to hook required action to load and process extra fields in the WC order related triggers.
	 * 
	 * @since 4.5.3
	 */
	public static function init() {
		// Extra options for workflow add to list action.
		add_action( 'ig_es_add_to_list_load_extra_fields', array( __CLASS__, 'add_product_list_fields' ) );
		
		// Perform add to product/variation list action.
		add_action( 'ig_es_after_action_run', array( __CLASS__, 'process_product_list_fields' ) );
	}

	/**
	 * Method to add extra fields to workflow add to list action.
	 *
	 * @param ES_Workflow_Action $action Action object
	 *
	 * @since 4.5.3
	 */
	public static function add_product_list_fields( $action ) {

		if ( $action instanceof ES_Workflow_Action && ! empty( $action->trigger ) ) {
			$trigger             = $action->trigger;
			$supplied_data_items = $trigger->get_supplied_data_items();

			// Check if this trigger provides WooCommerce order object.
			if ( in_array( 'wc_order', $supplied_data_items, true ) ) {

				$product_list_field = new ES_Checkbox();
				$product_list_field->set_name( 'ig-es-product-list-enabled' );
				$product_list_field->set_title( __( 'Add customer to product specific list', 'email-subscribers' ) );
				$product_list_field->add_classes( 'form-checkbox text-indigo-600' );
				$product_list_field->default_to_checked = false;

				$action->add_field( $product_list_field );

				$variation_list_field = new ES_Checkbox();
				$variation_list_field->set_name( 'ig-es-variation-list-enabled' );
				$variation_list_field->set_title( __( 'Add customer to product variation list', 'email-subscribers' ) );
				$variation_list_field->add_classes( 'form-checkbox text-indigo-600' );
				$variation_list_field->default_to_checked = false;

				$action->add_field( $variation_list_field );
			}
		}
	}

	/**
	 * Method to perform extra actions based on extra fields value.
	 * 
	 * @param ES_Worflow_Action $action Action object
	 * 
	 * @since 4.5.3
	 */
	public static function process_product_list_fields( $action ) {
		if ( $action instanceof ES_Action_Add_To_List ) {
			$raw_data = $action->workflow->data_layer()->get_raw_data();
			if ( ! empty( $raw_data ) ) {
				foreach ( $raw_data as $data_type_id => $data_item ) {
					$data_type = ES_Workflow_Data_Types::get( $data_type_id );
					// Check if we have WC_Order object.
					if ( $data_item instanceof WC_Order && is_callable( array( $data_item, 'get_items' ) ) ) {

						// Get product items from the order.
						$line_items = $data_item->get_items();

						if ( ! empty( $line_items ) ) {
							$product_list_enabled   = $action->get_option( 'ig-es-product-list-enabled' );
							$variation_list_enabled = $action->get_option( 'ig-es-variation-list-enabled' );
							$data                   = $data_type->get_data( $data_item );
							foreach ( $line_items as $line_item ) {
								$product = $line_item->get_product();

								if ( $product instanceof WC_Product ) {

									// If product is a variation then get its parent product.
									if ( $product->is_type( 'variation' ) ) {
										$parent_product_id = $product->get_parent_id();
										$parent_product    = wc_get_product( $parent_product_id );

										// Check if list also has to be created for variation product also.
										if ( $variation_list_enabled ) {
											$variation_list_id = ig_es_create_list_from_product( $product );
											if ( ! empty( $variation_list_id ) ) {
												$action->add_contact( $variation_list_id, $data );
											}
										}
									} else {
										$parent_product = $product;
									}
									
									if ( $product_list_enabled ) {
										$parent_list_id = ig_es_create_list_from_product( $parent_product );
										if ( ! empty( $parent_list_id ) ) {
											$action->add_contact( $parent_list_id, $data );
										}
									}
								}
							}
						}
					}
				}
			}
		}
	}
}

ES_Pro_Action_Add_To_List::init();
