<?php
/**
 * Workflow data type user
 *
 * @since       4.6.5
 * @version     1.0
 * @package     Email Subscribers
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Class to handle IG_ES_Customer data item
 *
 * @class IG_ES_Data_Type_Customer
 *
 * @since 4.6.5
 */
class IG_ES_Data_Type_Customer extends ES_Workflow_Data_Type {

	/**
	 * Validate given data item
	 *
	 * @since 4.6.5
	 *
	 * @param IG_ES_Customer $item Data item object.
	 *
	 * @return bool
	 */
	public function validate( $item ) {

		if ( ! ( $item instanceof IG_ES_Customer ) ) {
			return false;
		}

		if ( ! $item->get_email() ) {
			return false; // social login users may not have an email address defined
		}

		return true;
	}


	/**
	 * Returns id of given data item object. Only validated $items should be passed to this method
	 *
	 * @since 4.6.5
	 *
	 * @param IG_ES_Customer $item Data item object.
	 *
	 * @return mixed
	 */
	public function compress( $item ) {
		if ( $item->is_registered() ) {
			return $item->get_id();
		} else {
			$guest_id = $item->get_guest_id();
			// To distinguish guest user from registered user, store their id with 'guest_' prefix.
			return 'guest_' . $guest_id;
		}
	}


	/**
	 * Return data item object from given id.
	 *
	 * @since 4.6.5
	 *
	 * @param string $compressed_item Data item object ID.
	 * @param array  $compressed_data_layer Data layer.
	 *
	 * @return mixed
	 */
	public function decompress( $compressed_item, $compressed_data_layer ) {

		if ( $compressed_item ) {
			// IDs with 'guest_' prefix are of guest users.
			if ( false !== strpos( $compressed_item, 'guest_' ) ) {
				$guest_id = str_replace( 'guest_', '', $compressed_item );
				if ( ! empty( $guest_id ) && is_numeric( $guest_id ) ) {
					return IG_ES_Customer_Factory::get_by_guest_id( absint( $guest_id ) );
				}
			} else {
				return IG_ES_Customer_Factory::get( absint( $compressed_item ) );
			}
		}

		return false;
	}

	/**
	 * Abstract required data from data item object
	 *
	 * @since 4.6.5
	 *
	 * @param IG_ES_Customer $item Data item object.
	 * @return array
	 */
	public function get_data( $item ) {

		$data = array();

		if ( $item instanceof IG_ES_Customer ) {

			$first_name = '';
			$last_name  = '';
			
			if ( $item->is_registered() ) {
				$wp_user = $item->get_user();
				if ( $wp_user instanceof WP_User ) {
					$email      = $item->get_email();
					$first_name = $item->get_first_name();
					$last_name  = $item->get_last_name();
					$wp_user_id = $item->get_user_id();
				}
			} else {
				$guest = $item->get_guest();
				if ( $guest instanceof IG_ES_Guest ) {
					$email      = $guest->get_email();
					$first_name = $guest->get_first_name();
					$last_name  = $guest->get_last_name();
					$wp_user_id = 0;
				}
			}
			
			// prepare data.
			$data = array(
				'first_name' => $first_name,
				'last_name'  => $last_name,
				'email'      => $email,
				'source'     => 'woocommerce-abandoned-cart',
				'status'     => 'verified',
				'hash'       => ES_Common::generate_guid(),
				'created_at' => ig_get_current_date_time(),
				'wp_user_id' => $wp_user_id,
			);
		}

		return $data;
	}
}
