<?php

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Helper class to trigger products on sale event
 * 
 * @since 4.6.10
 */
class IG_ES_WC_Products_On_Sale {

	/**
	 * Hook class methods
	 */
	public static function init() {
		add_action( 'ig_es_wc_products_on_sale_worker', array( __CLASS__, 'check_for_new_products_on_sale' ) );
		add_action( 'ig_es_process_wc_wishlists', array( __CLASS__, 'process_wishlists' ) );
	}

	/**
	 * Check if there are any new products on sale
	 */
	public static function check_for_new_products_on_sale() {

		$last_on_sale = get_option( 'ig_es_wc_products_last_on_sale', array() );
		$now_on_sale  = wc_get_product_ids_on_sale();

		// Check if there are any new products which are on sale.
		$new_products_on_sale = array_diff( $now_on_sale, $last_on_sale );
		
		if ( $new_products_on_sale ) {

			$wishlist_ids = IG_ES_Wishlists::get_all_wishlist_ids();
			
			if ( ! empty( $wishlist_ids ) ) {
				
				update_option( 'ig_es_wc_wishlist_ids', $wishlist_ids );
				
				IG_ES_Background_Process_Helper::add_action_scheduler_task( 'ig_es_process_wc_wishlists' );
			}

			// Update option with current products which are on sale.
			update_option( 'ig_es_wc_products_last_on_sale', $now_on_sale, false );
		}
	}

	public static function process_wishlists() {

		$batch_start_time = time();

		/** 
		 * By subtracting the waiting time from $batch_start_time now, 
		 * We are allowing timeout to happen in the background process loop 3 seconds earlier.
		 * This earlier timeout will ensure we get engough time to make another asynchrounous request 
		 * since we need to wait for sometime before making the asynchronous request.
		 **/ 
		$batch_start_time = $batch_start_time - IG_ES_Background_Process_Helper::get_wait_seconds();

		$wishlist_ids = get_option( 'ig_es_wc_wishlist_ids', array() );
		
		$sale_product_ids = get_option( 'ig_es_wc_products_last_on_sale', array() );
		$trigger = ES_Workflow_Triggers::get( 'ig_es_wc_wishlist_item_goes_on_sale' );
		
		if ( empty( $wishlist_ids ) || empty( $sale_product_ids ) || ! $trigger || ! $trigger->has_workflows()  ) {
			return;
		}
		
		foreach ( $wishlist_ids as $index => $wishlist_id ) {
			
			$wishlist = IG_ES_Wishlists::get_wishlist( ES_Clean::id( $wishlist_id ) );

			// Remove the processed wishlist.
			unset( $wishlist_ids[ $index ] );
			
			$user_id = $wishlist->get_user_id();
			if ( ! $wishlist || ! $user_id ) {
				continue;
			}
			
			$wishlist_product_ids = $wishlist->get_items();
			$common_product_ids = array_intersect( $wishlist_product_ids, $sale_product_ids );
			
			// Check if this wishlist contains products which are currently on sale on not. If not ignore the wishlist.
			if ( empty( $common_product_ids ) ) {
				continue;
			}

			$customer = IG_ES_Customer_Factory::get_by_user_id( $wishlist->get_user_id() );

			if ( $customer instanceof IG_ES_Customer ) {
				do_action( 'ig_es_wc_wishlist_products_gone_on_sale', $wishlist, $customer );
			}
			
			// Check if time limit or memory limit has been reached.
			if ( IG_ES_Background_Process_Helper::time_exceeded( $batch_start_time ) || IG_ES_Background_Process_Helper::memory_exceeded() ) {
				break;
			}
		}

		// Check if there are still wishlist remaining. If yes then create another action scheduler task to process it.
		if ( ! empty( $wishlist_ids ) ) {
			update_option( 'ig_es_wc_wishlist_ids', $wishlist_ids );
			IG_ES_Background_Process_Helper::add_action_scheduler_task( 'ig_es_process_wc_wishlists' );
		} else {
			// Delete wishlist option if all of them has been processed.
			delete_option( 'ig_es_wc_wishlist_ids' );
		}
	}
}
