<?php
/**
 * Triggers when a user role get changed
 *
 * @since       4.6.6
 * @version     1.0
 * @package     Email Subscribers
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/***
 * ES_Trigger_User_Role_Changed class.
 *
 */
class ES_Trigger_User_Role_Changed extends ES_Workflow_Trigger {

	/**
	 * Declares data items available in trigger.
	 *
	 * @var array
	 */
	public $supplied_data_items = array( 'user' );

	/**
	 * Load trigger admin props.
	 */
	public function load_admin_details() {
		$this->title       = __( 'User Role Changed', 'email-subscribers' );
		$this->description = __( 'Fires when user role is changed.', 'email-subscribers' );
		$this->group       = __( 'User', 'email-subscribers' );
	}

	public function load_fields() {
		$user_roles = array();
			
		// Get only those roles which current user allowed to edit.
		$editable_roles = get_editable_roles();

		if ( ! empty( $editable_roles ) ) {
			foreach ( $editable_roles as $role_id => $role ) {
				$role_name              = translate_user_role( $role['name'] );
				$user_roles[ $role_id ] = $role_name;
			}
		}

		if ( ! empty( $user_roles ) ) {

			$old_user_roles_field = new ES_Select();
			$old_user_roles_field->set_name( 'ig-es-old-user-roles' );
			$old_user_roles_field->set_title( __( 'Old user roles', 'email-subscribers' ) );
			$old_user_roles_field->set_placeholder( __( 'Select user roles', 'email-subscribers' ) );
			$old_user_roles_field->set_description( __( 'Select old user roles for which this trigger should run. If left empty, trigger will run for all user roles.', 'email-subscribers' ) );
			$old_user_roles_field->add_classes( 'ig-es-form-multiselect' );
			$old_user_roles_field->set_options( $user_roles );
			$old_user_roles_field->set_multiple();
			$this->add_field( $old_user_roles_field );

			$new_user_roles_field = new ES_Select();
			$new_user_roles_field->set_name( 'ig-es-new-user-roles' );
			$new_user_roles_field->set_title( __( 'New user roles', 'email-subscribers' ) );
			$new_user_roles_field->set_placeholder( __( 'Select user roles', 'email-subscribers' ) );
			$new_user_roles_field->set_description( __( 'Select new user roles for which this trigger should run. If left empty, trigger will run for all user roles.', 'email-subscribers' ) );
			$new_user_roles_field->add_classes( 'ig-es-form-multiselect' );
			$new_user_roles_field->set_options( $user_roles );
			$new_user_roles_field->set_multiple();
			$this->add_field( $new_user_roles_field );
		}
	}

	/**
	 * Register trigger hooks.
	 */
	public function register_hooks() {
		add_action( 'set_user_role', array( $this, 'handle_user_role_changed' ), 10, 3 );
	}


	/**
	 * Catch user registered hook
	 *
	 * @param int      $user_id   The user ID.
	 * @param string   $new_role  The new role.
	 * @param string[] $old_roles An array of the user's previous roles.
	 */
	public function handle_user_role_changed( $user_id, $new_role, $old_roles ) {

		// Get user info.
		$user = get_userdata( $user_id );
		if ( ! ( $user instanceof WP_User ) ) {
			return;
		}

		$data = array(
			'user'      => $user,
			'old_roles' => $old_roles, // We aren't passing $new_role explicitly as it can be fetched from the $user object.
		);

		$this->maybe_run( $data );
	}


	/**
	 * Validate a workflow.
	 *
	 * @param ES_Workflow $workflow Workflow object.
	 *
	 * @return bool
	 */
	public function validate_workflow( $workflow ) {

		$user = $workflow->data_layer()->get_item( 'user' );
		
		if ( ! $user ) {
			return false;
		}
		
		$raw_data = $workflow->data_layer()->get_raw_data();
		$old_roles = ! empty( $raw_data['old_roles'] ) ? $raw_data['old_roles'] : array();
		if ( ! empty( $old_roles ) ) {
			$old_user_roles = $workflow->get_trigger_option( 'ig-es-old-user-roles' );
				
			// Check if admin has set user roles for this workflow and current user does not have that role. If yes then invalidate workflow.
			if ( ! empty( $old_user_roles ) && ! array_intersect( $old_roles, $old_user_roles ) ) {
				return false;
			}
		}

		$new_user_roles = $workflow->get_trigger_option( 'ig-es-new-user-roles' );
				
		// Check if admin has set user roles for this workflow and current user does not have that role. If yes then invalidate workflow.
		if ( ! empty( $new_user_roles ) && ! array_intersect( $user->roles, $new_user_roles ) ) {
			return false;
		}

		return true;
	}

}
