<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Class to trigger product review added workflow
 * 
 * @class ES_Trigger_WC_Product_Review_Approved
 * 
 * @since 4.5.3
 */
class ES_Trigger_WC_Product_Review_Approved extends ES_Workflow_Trigger {

	public $supplied_data_items = array( 'review', 'coupon', 'shop' );

	/**
	 * Load trigger admin details.
	 * 
	 * @since 4.5.3
	 */
	public function load_admin_details() {
		$this->title       = __( 'New Product Review Posted', 'email-subscribers' );
		$this->group       = __( 'Review', 'email-subscribers' );
		$this->description = __( 'This trigger does not fire until the review has been approved.', 'email-subscribers' );
	}

	/**
	 * Register required hooks to handle product review.
	 * 
	 * @since 4.5.3
	 */
	public function register_hooks() {
		add_action( 'wp_insert_comment', array( $this, 'handle_product_review_added' ), 10, 2 );
		add_action( 'transition_comment_status', array( $this, 'catch_review_approval' ), 10, 3 );
	}

	/**
	 * Handle product review added.
	 *
	 * @param int         $comment_id
	 * @param \WP_Comment $comment
	 * 
	 * @since 4.5.3
	 */
	public function handle_product_review_added( $comment_id, $comment ) {

		// Verify if it is a product review.
		if ( ! $comment || 'review' !== $comment->comment_type || 'product' !== get_post_type( $comment->comment_post_ID ) ) {
			return;
		}

		// Fetch user conset if given.
		$opt_in_consent      = ig_es_get_request_data( 'ig-es-comment-es-consent', 'no' );
		$show_opt_in_consent = get_site_option( 'ig_es_show_opt_in_consent', 'no' );
		
		// By default set consent status to default so when consent is enabled from settings we can get it from the consent field when submitting review.
		$consent_status = 'not_required';

		// Check consent only when user is not logged in since our consent field is shown in the comment form only when user is not logged in.
		// TODO: Remove ! is_user_logged_in() condition check when issue get fixed in the WordPress core.
		// Reference: https://core.trac.wordpress.org/ticket/16576
		if ( 'yes' === $show_opt_in_consent && ! is_user_logged_in() ) {
			$consent_status = ( 'yes' === $opt_in_consent ) ? 'given' : 'not_given';
		}

		// If user haven't given consent when consent field present then return.
		if ( 'not_given' === $consent_status ) {
			return;
		}

		// Store consent status in comment meta.
		update_comment_meta( $comment_id, 'ig_es_consent', $consent_status );
	}

	/**
	 * Trigger workflow if review has been approved.
	 * 
	 * @param string $new_status New review status.
	 * @param string $old_status Old review status.
	 * @param WP_Comment $comment Comment object.
	 * 
	 * @since 4.5.3
	 */
	public function catch_review_approval( $new_status, $old_status, $comment ) {

		// Verify if it is a product review and has been approved.
		if ( ! $comment || 'review' !== $comment->comment_type || 'approved' !== $new_status || 'product' !== get_post_type( $comment->comment_post_ID ) ) {
			return;
		}

		$consent_given = get_comment_meta( $comment->comment_ID, 'ig_es_consent', true );

		// Proceed only if consent was given when required at the time of adding the review.
		if ( ! in_array( $consent_given, array( 'given', 'not_required' ), true ) ) {
			return;
		}

		$customer = IG_ES_Customer_Factory::get_by_email( $comment->comment_author_email );

		$this->maybe_run(
			array(
				'review'   => $comment,
			)
		);
	}

	/**
	 * Validate workflow
	 * 
	 * @param $workflow Workflow
	 * @return bool
	 * 
	 * @since 4.5.3
	 */
	public function validate_workflow( $workflow ) {
		
		$review = $workflow->data_layer()->get_item( 'review' );

		// If workflow does not have data having type review then return.
		if ( ! $review ) {
			return false;
		}

		$email = ! empty( $review->comment_author_email ) ? $review->comment_author_email : '';

		// If email isn't provided then we can't proceed with the workflow.
		if ( empty( $email ) ) {
			return false;
		}

		return true;
	}
}
