<?php
/**
 * Adds extra fields to existing user registered trigger
 *
 * @since       4.5.3
 * @version     1.0
 * @package     Email Subscribers
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/***
 * ES_Pro_Trigger_User_Registered class.
 *
 * @since 4.5.3
 */
class ES_Pro_Trigger_User_Registered {
	
	/**
	 * Method to hook required action to load and process extra fields in the trigger 'ig_es_user_registered'.
	 * 
	 * @since 4.5.3
	 */
	public static function init() {
		// Extra options for workflow trigger.
		add_action( 'ig_es_user_registered_load_extra_fields', array( __CLASS__, 'load_extra_fields' ) );
		
		// Validate user registration worflows against user roles if set in the workflow.
		add_filter( 'ig_es_validate_workflow_ig_es_user_registered', array( __CLASS__, 'validate_user_registered_workflow' ), 10, 2 );
	}

	/**
	 * Method to add user roles field
	 * 
	 * @param ES_Workflow_Trigger $trigger_data Trigger data
	 * 
	 * @since 4.5.3
	 */
	public static function load_extra_fields( $trigger ) {

		if ( $trigger instanceof ES_Workflow_Trigger ) {

			$user_roles = array();
			
			// Get only those roles which current user allowed to edit.
			$editable_roles = get_editable_roles();

			if ( ! empty( $editable_roles ) ) {
				foreach ( $editable_roles as $role_id => $role ) {
					$role_name              = translate_user_role( $role['name'] );
					$user_roles[ $role_id ] = $role_name;
				}
			}

			if ( ! empty( $user_roles ) ) {
				$user_roles_field = new ES_Select();
				$user_roles_field->set_name( 'ig-es-allowed-user-roles' );
				$user_roles_field->set_title( __( 'User roles', 'email-subscribers' ) );
				$user_roles_field->set_placeholder( __( 'Select user roles', 'email-subscribers' ) );
				$user_roles_field->set_description( __( 'Select user roles for which this trigger should run. If left empty, trigger will run for all user roles.', 'email-subscribers' ) );
				$user_roles_field->add_classes( 'ig-es-form-multiselect' );
				$user_roles_field->set_options( $user_roles );
				$user_roles_field->set_multiple();
				$trigger->add_field( $user_roles_field );
			}
		}
	}

	/**
	 * Method to validate a workflow based on user registered trigger.
	 * 
	 * @param boolean $is_valid Is workflow valid.
	 * @param ES_Workflow $workflow Workflow object.
	 * 
	 * @return boolean $is_valid Is workflow valid.
	 * 
	 * @since 4.5.3
	 */
	public static function validate_user_registered_workflow( $is_valid, $workflow ) {

		if ( $workflow instanceof ES_Workflow ) {
			$user = $workflow->data_layer()->get_item( 'user' );
			if ( $user instanceof WP_User ) {
				$allowed_user_roles = $workflow->get_trigger_option( 'ig-es-allowed-user-roles' );
				
				// Check if admin has set user roles for this workflow and current user does not have that role. If yes then invalidate workflow.
				if ( ! empty( $allowed_user_roles ) && ! array_intersect( $user->roles, $allowed_user_roles ) ) {
					$is_valid = false;
				} else {
					// If the admin has not set the user roles in this workflow(existing workflows) or the current user has the required role then validate workflow.
					$is_valid = true;
				}
			}
		}

		return $is_valid;
	}
}

ES_Pro_Trigger_User_Registered::init();
